@@ -28,10 +28,10 @@
 
 
 **************************************************************************/
-#ifndef _P_EventSystem_h
-#define _P_EventSystem_h
+#pragma once
+#define _P_EventSystem_H
 
-#include "libts.h"
+#include "tscore/ink_platform.h"
 
 #include "I_EventSystem.h"
 
@@ -45,8 +45,6 @@
 #include "P_ProtectedQueue.h"
 #include "P_UnixEventProcessor.h"
 #include "P_UnixSocketManager.h"
-#undef EVENT_SYSTEM_MODULE_VERSION
-#define EVENT_SYSTEM_MODULE_VERSION \
-  makeModuleVersion(EVENT_SYSTEM_MODULE_MAJOR_VERSION, EVENT_SYSTEM_MODULE_MINOR_VERSION, PRIVATE_MODULE_HEADER)
 
-#endif
+static constexpr ts::ModuleVersion EVENT_SYSTEM_MODULE_INTERNAL_VERSION{EVENT_SYSTEM_MODULE_PUBLIC_VERSION,
+                                                                        ts::ModuleVersion::PRIVATE};
