@@ -25,7 +25,7 @@
 #include <I_EventSystem.h>
 #include <P_EventSystem.h> // TODO: less? just need ET_TASK
 
-#include "tscore/Map.h"
+#include "tscore/IntrusiveHashMap.h"
 #include "tscore/PriorityQueue.h"
 
 #include "tscore/List.h"
@@ -37,8 +37,10 @@
 #define REFCOUNT_CACHE_EVENT_SYNC REFCOUNT_CACHE_EVENT_EVENTS_START
 
 #define REFCOUNTCACHE_MAGIC_NUMBER 0x0BAD2D9
-#define REFCOUNTCACHE_MAJOR_VERSION 1
-#define REFCOUNTCACHE_MINOR_VERSION 0
+
+static constexpr unsigned char REFCOUNTCACHE_MAJOR_VERSION = 1;
+static constexpr unsigned char REFCOUNTCACHE_MINOR_VERSION = 0;
+static constexpr ts::VersionNumber REFCOUNTCACHE_VERSION(1, 0);
 
 // Stats
 enum RefCountCache_Stats {
@@ -73,12 +75,13 @@ class RefCountCacheHashEntry
 {
 public:
   Ptr<RefCountObj> item;
-  LINK(RefCountCacheHashEntry, item_link);
-  PriorityQueueEntry<RefCountCacheHashEntry *> *expiry_entry;
+  RefCountCacheHashEntry *_next{nullptr};
+  RefCountCacheHashEntry *_prev{nullptr};
+  PriorityQueueEntry<RefCountCacheHashEntry *> *expiry_entry = nullptr;
   RefCountCacheItemMeta meta;
 
   // Need a no-argument constructor to use the classAllocator
-  RefCountCacheHashEntry() : item(Ptr<RefCountObj>()), expiry_entry(nullptr), meta(0, 0) {}
+  RefCountCacheHashEntry() : item(Ptr<RefCountObj>()), meta(0, 0) {}
   void
   set(RefCountObj *i, uint64_t key, unsigned int size, int expire_time)
   {
@@ -113,24 +116,32 @@ public:
 // Since the hashing values are all fixed size, we can simply use a classAllocator to avoid mallocs
 extern ClassAllocator<PriorityQueueEntry<RefCountCacheHashEntry *>> expiryQueueEntry;
 
-struct RefCountCacheHashing {
-  typedef uint64_t ID;
-  typedef uint64_t const Key;
-  typedef RefCountCacheHashEntry Value;
-  typedef DList(RefCountCacheHashEntry, item_link) ListHead;
+struct RefCountCacheLinkage {
+  using key_type   = uint64_t const;
+  using value_type = RefCountCacheHashEntry;
 
-  static ID
-  hash(Key key)
+  static value_type *&
+  next_ptr(value_type *value)
+  {
+    return value->_next;
+  }
+  static value_type *&
+  prev_ptr(value_type *value)
+  {
+    return value->_prev;
+  }
+  static uint64_t
+  hash_of(key_type key)
   {
     return key;
   }
-  static Key
-  key(Value const *value)
+  static key_type
+  key_of(value_type *v)
   {
-    return value->meta.key;
+    return v->meta.key;
   }
   static bool
-  equal(Key lhs, Key rhs)
+  equal(key_type lhs, key_type rhs)
   {
     return lhs == rhs;
   }
@@ -141,6 +152,8 @@ struct RefCountCacheHashing {
 template <class C> class RefCountCachePartition
 {
 public:
+  using hash_type = IntrusiveHashMap<RefCountCacheLinkage>;
+
   RefCountCachePartition(unsigned int part_num, uint64_t max_size, unsigned int max_items, RecRawStatBlock *rsb = nullptr);
   Ptr<C> get(uint64_t key);
   void put(uint64_t key, C *item, int size = 0, int expire_time = 0);
@@ -149,15 +162,12 @@ public:
   void clear();
   bool is_full() const;
   bool make_space_for(unsigned int);
-  template <class Iterator> void dealloc_entry(Iterator ptr);
+  void dealloc_entry(hash_type::iterator ptr);
 
   size_t count() const;
   void copy(std::vector<RefCountCacheHashEntry *> &items);
 
-  typedef typename TSHashTable<RefCountCacheHashing>::iterator iterator_type;
-  typedef typename TSHashTable<RefCountCacheHashing>::self hash_type;
-  typedef typename TSHashTable<RefCountCacheHashing>::Location location_type;
-  TSHashTable<RefCountCacheHashing> *get_map();
+  hash_type &get_map();
 
   Ptr<ProxyMutex> lock; // Lock
 
@@ -188,11 +198,10 @@ Ptr<C>
 RefCountCachePartition<C>::get(uint64_t key)
 {
   this->metric_inc(refcountcache_total_lookups_stat, 1);
-  location_type l = this->item_map.find(key);
-  if (l.isValid()) {
+  if (auto it = this->item_map.find(key); it != this->item_map.end()) {
     // found
     this->metric_inc(refcountcache_total_hits_stat, 1);
-    return make_ptr((C *)l.m_value->item.get());
+    return make_ptr(static_cast<C *>(it->item.get()));
   } else {
     return Ptr<C>();
   }
@@ -239,44 +248,37 @@ template <class C>
 void
 RefCountCachePartition<C>::erase(uint64_t key, ink_time_t expiry_time)
 {
-  location_type l = this->item_map.find(key);
-  if (l.isValid()) {
-    if (expiry_time >= 0 && l.m_value->meta.expiry_time != expiry_time) {
+  if (auto it = this->item_map.find(key); it != this->item_map.end()) {
+    if (expiry_time >= 0 && it->meta.expiry_time != expiry_time) {
       return;
     }
-
-    // TSHashMap does NOT clean up the item-- this remove just removes it from the map
-    // we are responsible for cleaning it up here
-    this->item_map.remove(l);
-    this->dealloc_entry(l);
+    this->item_map.erase(it);
+    this->dealloc_entry(it);
   }
 }
 
 template <class C>
-template <class Iterator>
 void
-RefCountCachePartition<C>::dealloc_entry(Iterator ptr)
+RefCountCachePartition<C>::dealloc_entry(hash_type::iterator ptr)
 {
-  if (ptr.m_value) {
-    // decrement usag are not cleaned up. The values are not touched in this method, therefore it is safe
-    // counters
-    this->size -= ptr->meta.size;
-    this->items--;
+  // decrement usage are not cleaned up. The values are not touched in this method, therefore it is safe
+  // counters
+  this->size -= ptr->meta.size;
+  this->items--;
 
-    this->metric_inc(refcountcache_current_size_stat, -((int64_t)ptr->meta.size));
-    this->metric_inc(refcountcache_current_items_stat, -1);
+  this->metric_inc(refcountcache_current_size_stat, -((int64_t)ptr->meta.size));
+  this->metric_inc(refcountcache_current_items_stat, -1);
 
-    // remove from expiry queue
-    if (ptr->expiry_entry != nullptr) {
-      Debug("refcountcache", "partition %d deleting item from expiry_queue idx=%d", this->part_num, ptr->expiry_entry->index);
+  // remove from expiry queue
+  if (ptr->expiry_entry != nullptr) {
+    Debug("refcountcache", "partition %d deleting item from expiry_queue idx=%d", this->part_num, ptr->expiry_entry->index);
 
-      this->expiry_queue.erase(ptr->expiry_entry);
-      expiryQueueEntry.free(ptr->expiry_entry);
-      ptr->expiry_entry = nullptr; // To avoid the destruction of `l` calling the destructor again-- and causing issues
-    }
-
-    RefCountCacheHashEntry::free<C>(ptr.m_value);
+    this->expiry_queue.erase(ptr->expiry_entry);
+    expiryQueueEntry.free(ptr->expiry_entry);
+    ptr->expiry_entry = nullptr; // To avoid the destruction of `l` calling the destructor again-- and causing issues
   }
+
+  RefCountCacheHashEntry::free<C>(ptr);
 }
 
 template <class C>
@@ -286,12 +288,12 @@ RefCountCachePartition<C>::clear()
   // Since the hash nodes embed the list pointers, you can't iterate over the
   // hash elements and deallocate them, let alone remove them from the hash.
   // Hence, this monstrosity.
-  while (this->item_map.count() > 0) {
-    location_type pos = this->item_map.find(this->item_map.begin().m_value);
+  auto it = this->item_map.begin();
+  while (it != this->item_map.end()) {
+    auto cur = it++;
 
-    ink_assert(pos.isValid());
-    this->item_map.remove(pos);
-    this->dealloc_entry(pos);
+    this->item_map.erase(cur);
+    this->dealloc_entry(cur);
   }
 }
 
@@ -339,9 +341,9 @@ template <class C>
 void
 RefCountCachePartition<C>::copy(std::vector<RefCountCacheHashEntry *> &items)
 {
-  for (RefCountCachePartition<C>::iterator_type i = this->item_map.begin(); i != this->item_map.end(); ++i) {
+  for (auto &&it : this->item_map) {
     RefCountCacheHashEntry *val = RefCountCacheHashEntry::alloc();
-    val->set(i.m_value->item.get(), i.m_value->meta.key, i.m_value->meta.size, i.m_value->meta.expiry_time);
+    val->set(it.item.get(), it.meta.key, it.meta.size, it.meta.expiry_time);
     items.push_back(val);
   }
 }
@@ -356,23 +358,23 @@ RefCountCachePartition<C>::metric_inc(RefCountCache_Stats metric_enum, int64_t d
 }
 
 template <class C>
-TSHashTable<RefCountCacheHashing> *
+IntrusiveHashMap<RefCountCacheLinkage> &
 RefCountCachePartition<C>::get_map()
 {
-  return &this->item_map;
+  return this->item_map;
 }
 
 // The header for the cache, this is used to check if the serialized cache is compatible
 class RefCountCacheHeader
 {
 public:
-  unsigned int magic;
-  VersionNumber version;
-  VersionNumber object_version; // version passed in of whatever it is we are caching
+  unsigned int magic = REFCOUNTCACHE_MAGIC_NUMBER;
+  ts::VersionNumber version{REFCOUNTCACHE_VERSION};
+  ts::VersionNumber object_version; // version passed in of whatever it is we are caching
 
-  RefCountCacheHeader(VersionNumber object_version = VersionNumber());
-  bool operator==(const RefCountCacheHeader other) const;
-  bool compatible(RefCountCacheHeader *other) const;
+  RefCountCacheHeader(ts::VersionNumber object_version = ts::VersionNumber());
+  bool operator==(RefCountCacheHeader const &that) const;
+  bool compatible(RefCountCacheHeader *that) const;
 };
 
 // RefCountCache is a ref-counted key->value map to store classes that inherit from RefCountObj.
@@ -393,8 +395,8 @@ template <class C> class RefCountCache
 {
 public:
   // Constructor
-  RefCountCache(unsigned int num_partitions, int size = -1, int items = -1, VersionNumber object_version = VersionNumber(),
-                std::string metrics_prefix = "");
+  RefCountCache(unsigned int num_partitions, int size = -1, int items = -1, ts::VersionNumber object_version = ts::VersionNumber(),
+                const std::string &metrics_prefix = "");
   // Destructor
   ~RefCountCache();
 
@@ -406,7 +408,7 @@ public:
 
   // Some methods to get some internal state
   int partition_for_key(uint64_t key);
-  ProxyMutex *lock_for_key(uint64_t key);
+  Ptr<ProxyMutex> lock_for_key(uint64_t key);
   size_t partition_count() const;
   RefCountCachePartition<C> &get_partition(int pnum);
   size_t count() const;
@@ -424,8 +426,8 @@ private:
 };
 
 template <class C>
-RefCountCache<C>::RefCountCache(unsigned int num_partitions, int size, int items, VersionNumber object_version,
-                                std::string metrics_prefix)
+RefCountCache<C>::RefCountCache(unsigned int num_partitions, int size, int items, ts::VersionNumber object_version,
+                                const std::string &metrics_prefix)
   : header(RefCountCacheHeader(object_version)), rsb(nullptr)
 {
   this->max_size       = size;
@@ -508,10 +510,10 @@ RefCountCache<C>::get_header()
 }
 
 template <class C>
-ProxyMutex *
+Ptr<ProxyMutex>
 RefCountCache<C>::lock_for_key(uint64_t key)
 {
-  return this->partitions[this->partition_for_key(key)]->lock.get();
+  return this->partitions[this->partition_for_key(key)]->lock;
 }
 
 template <class C>
@@ -566,7 +568,7 @@ RefCountCache<C>::clear()
 // Errors are -1
 template <typename CacheEntryType>
 int
-LoadRefCountCacheFromPath(RefCountCache<CacheEntryType> &cache, std::string dirname, std::string filepath,
+LoadRefCountCacheFromPath(RefCountCache<CacheEntryType> &cache, const std::string &dirname, const std::string &filepath,
                           CacheEntryType *(*load_func)(char *, unsigned int))
 {
   // If we have no load method, then we can't load anything so lets just stop right here
