@@ -23,16 +23,25 @@
 
 #include "P_RecCore.h"
 #include "P_RecProcess.h"
+#include <string_view>
 
 //-------------------------------------------------------------------------
 // raw_stat_get_total
 //-------------------------------------------------------------------------
+
+namespace
+{
+// Commonly used access to a raw stat, avoid typos.
+inline RecRawStat *
+thread_stat(EThread *et, RecRawStatBlock *rsb, int id)
+{
+  return (reinterpret_cast<RecRawStat *>(reinterpret_cast<char *>(et) + rsb->ethr_stat_offset)) + id;
+}
+} // namespace
+
 static int
 raw_stat_get_total(RecRawStatBlock *rsb, int id, RecRawStat *total)
 {
-  int i;
-  RecRawStat *tlp;
-
   total->sum   = 0;
   total->count = 0;
 
@@ -41,14 +50,14 @@ raw_stat_get_total(RecRawStatBlock *rsb, int id, RecRawStat *total)
   total->count = rsb->global[id]->count;
 
   // get thread local values
-  for (i = 0; i < eventProcessor.n_ethreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_ethreads[i]) + rsb->ethr_stat_offset)) + id;
+  for (EThread *et : eventProcessor.active_ethreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
     total->sum += tlp->sum;
     total->count += tlp->count;
   }
 
-  for (i = 0; i < eventProcessor.n_dthreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_dthreads[i]) + rsb->ethr_stat_offset)) + id;
+  for (EThread *et : eventProcessor.active_dthreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
     total->sum += tlp->sum;
     total->count += tlp->count;
   }
@@ -66,22 +75,20 @@ raw_stat_get_total(RecRawStatBlock *rsb, int id, RecRawStat *total)
 static int
 raw_stat_sync_to_global(RecRawStatBlock *rsb, int id)
 {
-  int i;
-  RecRawStat *tlp;
   RecRawStat total;
 
   total.sum   = 0;
   total.count = 0;
 
   // sum the thread local values
-  for (i = 0; i < eventProcessor.n_ethreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_ethreads[i]) + rsb->ethr_stat_offset)) + id;
+  for (EThread *et : eventProcessor.active_ethreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
     total.sum += tlp->sum;
     total.count += tlp->count;
   }
 
-  for (i = 0; i < eventProcessor.n_dthreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_dthreads[i]) + rsb->ethr_stat_offset)) + id;
+  for (EThread *et : eventProcessor.active_dthreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
     total.sum += tlp->sum;
     total.count += tlp->count;
   }
@@ -91,27 +98,22 @@ raw_stat_sync_to_global(RecRawStatBlock *rsb, int id)
   }
 
   // lock so the setting of the globals and last values are atomic
-  ink_mutex_acquire(&(rsb->mutex));
+  {
+    ink_scoped_mutex_lock lock(rsb->mutex);
 
-  // get the delta from the last sync
-  RecRawStat delta;
-  delta.sum   = total.sum - rsb->global[id]->last_sum;
-  delta.count = total.count - rsb->global[id]->last_count;
+    // get the delta from the last sync
+    RecRawStat delta;
+    delta.sum   = total.sum - rsb->global[id]->last_sum;
+    delta.count = total.count - rsb->global[id]->last_count;
 
-  // This is too verbose now, so leaving it out / leif
-  // Debug("stats", "raw_stat_sync_to_global(): rsb pointer:%p id:%d delta:%" PRId64 " total:%" PRId64 " last:%" PRId64 " global:%"
-  // PRId64 "\n",
-  // rsb, id, delta.sum, total.sum, rsb->global[id]->last_sum, rsb->global[id]->sum);
+    // increment the global values by the delta
+    ink_atomic_increment(&(rsb->global[id]->sum), delta.sum);
+    ink_atomic_increment(&(rsb->global[id]->count), delta.count);
 
-  // increment the global values by the delta
-  ink_atomic_increment(&(rsb->global[id]->sum), delta.sum);
-  ink_atomic_increment(&(rsb->global[id]->count), delta.count);
-
-  // set the new totals as the last values seen
-  ink_atomic_swap(&(rsb->global[id]->last_sum), total.sum);
-  ink_atomic_swap(&(rsb->global[id]->last_count), total.count);
-
-  ink_mutex_release(&(rsb->mutex));
+    // set the new totals as the last values seen
+    ink_atomic_swap(&(rsb->global[id]->last_sum), total.sum);
+    ink_atomic_swap(&(rsb->global[id]->last_count), total.count);
+  }
 
   return REC_ERR_OKAY;
 }
@@ -126,25 +128,24 @@ raw_stat_clear(RecRawStatBlock *rsb, int id)
 
   // the globals need to be reset too
   // lock so the setting of the globals and last values are atomic
-  ink_mutex_acquire(&(rsb->mutex));
-  ink_atomic_swap(&(rsb->global[id]->sum), (int64_t)0);
-  ink_atomic_swap(&(rsb->global[id]->last_sum), (int64_t)0);
-  ink_atomic_swap(&(rsb->global[id]->count), (int64_t)0);
-  ink_atomic_swap(&(rsb->global[id]->last_count), (int64_t)0);
-  ink_mutex_release(&(rsb->mutex));
-
+  {
+    ink_scoped_mutex_lock lock(rsb->mutex);
+    ink_atomic_swap(&(rsb->global[id]->sum), static_cast<int64_t>(0));
+    ink_atomic_swap(&(rsb->global[id]->last_sum), static_cast<int64_t>(0));
+    ink_atomic_swap(&(rsb->global[id]->count), static_cast<int64_t>(0));
+    ink_atomic_swap(&(rsb->global[id]->last_count), static_cast<int64_t>(0));
+  }
   // reset the local stats
-  RecRawStat *tlp;
-  for (int i = 0; i < eventProcessor.n_ethreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_ethreads[i]) + rsb->ethr_stat_offset)) + id;
-    ink_atomic_swap(&(tlp->sum), (int64_t)0);
-    ink_atomic_swap(&(tlp->count), (int64_t)0);
+  for (EThread *et : eventProcessor.active_ethreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
+    ink_atomic_swap(&(tlp->sum), static_cast<int64_t>(0));
+    ink_atomic_swap(&(tlp->count), static_cast<int64_t>(0));
   }
 
-  for (int i = 0; i < eventProcessor.n_dthreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_dthreads[i]) + rsb->ethr_stat_offset)) + id;
-    ink_atomic_swap(&(tlp->sum), (int64_t)0);
-    ink_atomic_swap(&(tlp->count), (int64_t)0);
+  for (EThread *et : eventProcessor.active_dthreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
+    ink_atomic_swap(&(tlp->sum), static_cast<int64_t>(0));
+    ink_atomic_swap(&(tlp->count), static_cast<int64_t>(0));
   }
 
   return REC_ERR_OKAY;
@@ -160,21 +161,21 @@ raw_stat_clear_sum(RecRawStatBlock *rsb, int id)
 
   // the globals need to be reset too
   // lock so the setting of the globals and last values are atomic
-  ink_mutex_acquire(&(rsb->mutex));
-  ink_atomic_swap(&(rsb->global[id]->sum), (int64_t)0);
-  ink_atomic_swap(&(rsb->global[id]->last_sum), (int64_t)0);
-  ink_mutex_release(&(rsb->mutex));
-
-  // reset the local stats
-  RecRawStat *tlp;
-  for (int i = 0; i < eventProcessor.n_ethreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_ethreads[i]) + rsb->ethr_stat_offset)) + id;
-    ink_atomic_swap(&(tlp->sum), (int64_t)0);
+  {
+    ink_scoped_mutex_lock lock(rsb->mutex);
+    ink_atomic_swap(&(rsb->global[id]->sum), static_cast<int64_t>(0));
+    ink_atomic_swap(&(rsb->global[id]->last_sum), static_cast<int64_t>(0));
   }
 
-  for (int i = 0; i < eventProcessor.n_dthreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_dthreads[i]) + rsb->ethr_stat_offset)) + id;
-    ink_atomic_swap(&(tlp->sum), (int64_t)0);
+  // reset the local stats
+  for (EThread *et : eventProcessor.active_ethreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
+    ink_atomic_swap(&(tlp->sum), static_cast<int64_t>(0));
+  }
+
+  for (EThread *et : eventProcessor.active_dthreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
+    ink_atomic_swap(&(tlp->sum), static_cast<int64_t>(0));
   }
 
   return REC_ERR_OKAY;
@@ -190,21 +191,21 @@ raw_stat_clear_count(RecRawStatBlock *rsb, int id)
 
   // the globals need to be reset too
   // lock so the setting of the globals and last values are atomic
-  ink_mutex_acquire(&(rsb->mutex));
-  ink_atomic_swap(&(rsb->global[id]->count), (int64_t)0);
-  ink_atomic_swap(&(rsb->global[id]->last_count), (int64_t)0);
-  ink_mutex_release(&(rsb->mutex));
-
-  // reset the local stats
-  RecRawStat *tlp;
-  for (int i = 0; i < eventProcessor.n_ethreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_ethreads[i]) + rsb->ethr_stat_offset)) + id;
-    ink_atomic_swap(&(tlp->count), (int64_t)0);
+  {
+    ink_scoped_mutex_lock lock(rsb->mutex);
+    ink_atomic_swap(&(rsb->global[id]->count), static_cast<int64_t>(0));
+    ink_atomic_swap(&(rsb->global[id]->last_count), static_cast<int64_t>(0));
   }
 
-  for (int i = 0; i < eventProcessor.n_dthreads; i++) {
-    tlp = ((RecRawStat *)((char *)(eventProcessor.all_dthreads[i]) + rsb->ethr_stat_offset)) + id;
-    ink_atomic_swap(&(tlp->count), (int64_t)0);
+  // reset the local stats
+  for (EThread *et : eventProcessor.active_ethreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
+    ink_atomic_swap(&(tlp->count), static_cast<int64_t>(0));
+  }
+
+  for (EThread *et : eventProcessor.active_dthreads()) {
+    RecRawStat *tlp = thread_stat(et, rsb, id);
+    ink_atomic_swap(&(tlp->count), static_cast<int64_t>(0));
   }
 
   return REC_ERR_OKAY;
@@ -225,17 +226,17 @@ RecAllocateRawStatBlock(int num_stats)
   }
 
   // create the raw-stat-block structure
-  rsb = (RecRawStatBlock *)ats_malloc(sizeof(RecRawStatBlock));
+  rsb = static_cast<RecRawStatBlock *>(ats_malloc(sizeof(RecRawStatBlock)));
   memset(rsb, 0, sizeof(RecRawStatBlock));
 
-  rsb->global = (RecRawStat **)ats_malloc(num_stats * sizeof(RecRawStat *));
+  rsb->global = static_cast<RecRawStat **>(ats_malloc(num_stats * sizeof(RecRawStat *)));
   memset(rsb->global, 0, num_stats * sizeof(RecRawStat *));
 
   rsb->num_stats        = 0;
   rsb->max_stats        = num_stats;
   rsb->ethr_stat_offset = ethr_stat_offset;
 
-  ink_mutex_init(&(rsb->mutex), "net stat mutex");
+  ink_mutex_init(&(rsb->mutex));
   return rsb;
 }
 
@@ -329,7 +330,7 @@ RecRawStatSyncAvg(const char *name, RecDataT data_type, RecData *data, RecRawSta
   total.sum   = rsb->global[id]->sum;
   total.count = rsb->global[id]->count;
   if (total.count != 0) {
-    avg = (float)((double)total.sum / (double)total.count);
+    avg = static_cast<float>(static_cast<double>(total.sum) / static_cast<double>(total.count));
   }
   RecDataSetFromFloat(data_type, data, avg);
   return REC_ERR_OKAY;
@@ -349,8 +350,8 @@ RecRawStatSyncHrTimeAvg(const char *name, RecDataT data_type, RecData *data, Rec
   if (total.count == 0) {
     r = 0.0f;
   } else {
-    r = (float)((double)total.sum / (double)total.count);
-    r = r / (float)(HRTIME_SECOND);
+    r = static_cast<float>(static_cast<double>(total.sum) / static_cast<double>(total.count));
+    r = r / static_cast<float>(HRTIME_SECOND);
   }
 
   RecDataSetFromFloat(data_type, data, r);
@@ -371,29 +372,7 @@ RecRawStatSyncIntMsecsToFloatSeconds(const char *name, RecDataT data_type, RecDa
   if (total.count == 0) {
     r = 0.0f;
   } else {
-    r = (float)((double)total.sum / 1000);
-  }
-
-  RecDataSetFromFloat(data_type, data, r);
-  return REC_ERR_OKAY;
-}
-
-int
-RecRawStatSyncMHrTimeAvg(const char *name, RecDataT data_type, RecData *data, RecRawStatBlock *rsb, int id)
-{
-  RecRawStat total;
-  RecFloat r;
-
-  Debug("stats", "raw sync:mhr-timeavg for %s", name);
-  raw_stat_sync_to_global(rsb, id);
-  total.sum   = rsb->global[id]->sum;
-  total.count = rsb->global[id]->count;
-
-  if (total.count == 0) {
-    r = 0.0f;
-  } else {
-    r = (float)((double)total.sum / (double)total.count);
-    r = r / (float)(HRTIME_MSECOND);
+    r = static_cast<float>(static_cast<double>(total.sum) / 1000);
   }
 
   RecDataSetFromFloat(data_type, data, r);
@@ -530,11 +509,11 @@ int
 RecRegisterRawStatSyncCb(const char *name, RecRawStatSyncCb sync_cb, RecRawStatBlock *rsb, int id)
 {
   int err = REC_ERR_FAIL;
-  RecRecord *r;
 
   ink_rwlock_rdlock(&g_records_rwlock);
+  if (auto it = g_records_ht.find(name); it != g_records_ht.end()) {
+    RecRecord *r = it->second;
 
-  if (ink_hash_table_lookup(g_records_ht, name, (void **)&r)) {
     rec_mutex_acquire(&(r->lock));
     if (REC_TYPE_IS_STAT(r->rec_type)) {
       if (r->stat_meta.sync_cb) {
@@ -597,9 +576,7 @@ RecRawStatUpdateSum(RecRawStatBlock *rsb, int id)
 {
   RecRawStat *raw = rsb->global[id];
   if (nullptr != raw) {
-    RecRecord *r = reinterpret_cast<RecRecord *>(reinterpret_cast<char *>(raw) -
-                                                 (reinterpret_cast<char *>(&reinterpret_cast<RecRecord *>(0)->stat_meta) -
-                                                  reinterpret_cast<char *>(reinterpret_cast<RecRecord *>(0))));
+    RecRecord *r = reinterpret_cast<RecRecord *>(reinterpret_cast<char *>(raw) - offsetof(struct RecRecord, stat_meta));
 
     RecDataSetFromInt64(r->data_type, &r->data, rsb->global[id]->sum);
     r->sync_required = REC_SYNC_REQUIRED;
