@@ -55,8 +55,6 @@ constexpr char PLUGIN[] = "statichit";
   VDEBUG("vio=%p vio.cont=%p, vio.cont.data=%p, vio.vc=%p " fmt, (vio), TSVIOContGet(vio), TSContDataGet(TSVIOContGet(vio)), \
          TSVIOVConnGet(vio), ##__VA_ARGS__)
 
-static TSCont TxnHook;
-
 static int StatCountBytes     = -1;
 static int StatCountResponses = -1;
 
@@ -64,9 +62,12 @@ static int StaticHitInterceptHook(TSCont contp, TSEvent event, void *edata);
 static int StaticHitTxnHook(TSCont contp, TSEvent event, void *edata);
 
 struct StaticHitConfig {
-  explicit StaticHitConfig(const std::string &filePath, const std::string &mimeType) : filePath(filePath), mimeType(mimeType) {}
+  explicit StaticHitConfig(const std::string &filePath, const std::string &mimeType, bool disableExact)
+    : filePath(filePath), mimeType(mimeType), disableExact(disableExact)
+  {
+  }
 
-  ~StaticHitConfig() {}
+  ~StaticHitConfig() { TSContDestroy(cont); }
 
   std::string filePath;
   std::string mimeType;
@@ -74,6 +75,10 @@ struct StaticHitConfig {
   int successCode = 200;
   int failureCode = 404;
   int maxAge      = 0;
+
+  bool disableExact = false;
+
+  TSCont cont;
 };
 
 struct StaticHitRequest;
@@ -361,7 +366,6 @@ StaticHitInterceptHook(TSCont contp, TSEvent event, void *edata)
     VDEBUG("reading vio=%p vc=%p, trq=%p", arg.vio, TSVIOVConnGet(arg.vio), cdata.trq);
 
     TSIOBufferBlock blk;
-    ssize_t consumed     = 0;
     TSParseResult result = TS_PARSE_CONT;
 
     for (blk = TSIOBufferReaderStart(cdata.trq->readio.reader); blk; blk = TSIOBufferBlockNext(blk)) {
@@ -403,8 +407,7 @@ StaticHitInterceptHook(TSCont contp, TSEvent event, void *edata)
         return TS_EVENT_NONE;
 
       case TS_PARSE_CONT:
-        // We consumed the buffer we got minus the remainder.
-        consumed += (nbytes - std::distance(ptr, end));
+        break;
       }
     }
 
@@ -522,7 +525,7 @@ StaticHitTxnHook(TSCont contp, TSEvent event, void *edata)
     }
 
     method = TSHttpHdrMethodGet(bufp, hdr_loc, &method_length);
-    if (NULL == method) {
+    if (nullptr == method) {
       VERROR("Couldn't retrieve client request method");
       goto done;
     }
@@ -547,8 +550,6 @@ done:
 TSReturnCode
 TSRemapInit(TSRemapInterface * /* api_info */, char * /* errbuf */, int /* errbuf_size */)
 {
-  TxnHook = TSContCreate(StaticHitTxnHook, nullptr);
-
   if (TSStatFindName("statichit.response_bytes", &StatCountBytes) == TS_ERROR) {
     StatCountBytes = TSStatCreate("statichit.response_bytes", TS_RECORDDATATYPE_COUNTER, TS_STAT_NON_PERSISTENT, TS_STAT_SYNC_SUM);
   }
@@ -563,6 +564,12 @@ TSRemapInit(TSRemapInterface * /* api_info */, char * /* errbuf */, int /* errbu
 TSRemapStatus
 TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
 {
+  const TSHttpStatus txnstat = TSHttpTxnStatusGet(rh);
+  if (txnstat != TS_HTTP_STATUS_NONE && txnstat != TS_HTTP_STATUS_OK) {
+    VDEBUG("transaction status_code=%d already set; skipping processing", static_cast<int>(txnstat));
+    return TSREMAP_NO_REMAP;
+  }
+
   StaticHitConfig *cfg = static_cast<StaticHitConfig *>(ih);
 
   if (!cfg) {
@@ -571,22 +578,23 @@ TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
     return TSREMAP_NO_REMAP;
   }
 
-  // Anchor to URL specified in remap
-  int pathsz;
-  TSUrlPathGet(rri->requestBufp, rri->requestUrl, &pathsz);
-  if (pathsz > 0) {
-    VERROR("Path is not an exact match. Rejecting!");
-    TSHttpTxnStatusSet(rh, TS_HTTP_STATUS_NOT_FOUND);
-    return TSREMAP_NO_REMAP;
+  if (!cfg->disableExact) {
+    // Anchor to URL specified in remap
+    int pathsz;
+    TSUrlPathGet(rri->requestBufp, rri->requestUrl, &pathsz);
+    if (pathsz > 0) {
+      VERROR("Path is not an exact match. Rejecting!");
+      TSHttpTxnStatusSet(rh, TS_HTTP_STATUS_NOT_FOUND);
+      return TSREMAP_NO_REMAP;
+    }
   }
 
   if (!cfg->maxAge) {
     TSHttpTxnConfigIntSet(rh, TS_CONFIG_HTTP_CACHE_HTTP, 0);
     StaticHitSetupIntercept(static_cast<StaticHitConfig *>(ih), rh);
   } else {
-    TSHttpTxnHookAdd(rh, TS_HTTP_CACHE_LOOKUP_COMPLETE_HOOK, TxnHook);
+    TSHttpTxnHookAdd(rh, TS_HTTP_CACHE_LOOKUP_COMPLETE_HOOK, static_cast<StaticHitConfig *>(ih)->cont);
   }
-  TSContDataSet(TxnHook, ih);
 
   return TSREMAP_NO_REMAP; // This plugin never rewrites anything.
 }
@@ -594,14 +602,18 @@ TSRemapDoRemap(void *ih, TSHttpTxn rh, TSRemapRequestInfo *rri)
 TSReturnCode
 TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSED */, int /* errbuf_size ATS_UNUSED */)
 {
-  static const struct option longopt[] = {
-    {"file-path", required_argument, NULL, 'f'},    {"mime-type", required_argument, NULL, 'm'},
-    {"max-age", required_argument, NULL, 'a'},      {"failure-code", required_argument, NULL, 'c'},
-    {"success-code", required_argument, NULL, 's'}, {NULL, no_argument, NULL, '\0'}};
+  static const struct option longopt[] = {{"file-path", required_argument, nullptr, 'f'},
+                                          {"mime-type", required_argument, nullptr, 'm'},
+                                          {"max-age", required_argument, nullptr, 'a'},
+                                          {"failure-code", required_argument, nullptr, 'c'},
+                                          {"success-code", required_argument, nullptr, 's'},
+                                          {"disable-exact", no_argument, nullptr, 'd'},
+                                          {nullptr, no_argument, nullptr, '\0'}};
 
   std::string filePath;
   std::string mimeType = "text/plain";
   int maxAge = 0, failureCode = 0, successCode = 0;
+  bool disableExact = false;
 
   // argv contains the "to" and "from" URLs. Skip the first so that the
   // second one poses as the program name.
@@ -610,7 +622,7 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
   optind = 0;
 
   while (true) {
-    int opt = getopt_long(argc, (char *const *)argv, "f:m:a:c:s:", longopt, NULL);
+    int opt = getopt_long(argc, (char *const *)argv, "f:m:a:c:s:d", longopt, nullptr);
 
     switch (opt) {
     case 'f': {
@@ -628,6 +640,9 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
     case 's': {
       successCode = atoi(optarg);
     } break;
+    case 'd': {
+      disableExact = true;
+    } break;
     }
 
     if (opt == -1) {
@@ -640,11 +655,11 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
     return TS_ERROR;
   }
 
-  if (filePath.find("/") != 0) {
+  if (filePath.find('/') != 0) {
     filePath = std::string(TSConfigDirGet()) + '/' + filePath;
   }
 
-  StaticHitConfig *tc = new StaticHitConfig(filePath, mimeType);
+  StaticHitConfig *tc = new StaticHitConfig(filePath, mimeType, disableExact);
   if (maxAge > 0) {
     tc->maxAge = maxAge;
   }
@@ -655,6 +670,10 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
     tc->successCode = successCode;
   }
 
+  // Finally, create the continuation to use for this remap rule, tracking the config as cont data.
+  tc->cont = TSContCreate(StaticHitTxnHook, nullptr);
+  TSContDataSet(tc->cont, tc);
+
   *ih = static_cast<void *>(tc);
 
   return TS_SUCCESS;
