@@ -26,14 +26,14 @@ Statement::append(Statement *stmt)
 {
   Statement *tmp = this;
 
-  TSReleaseAssert(stmt->_next == NULL);
+  TSReleaseAssert(stmt->_next == nullptr);
   while (tmp->_next) {
     tmp = tmp->_next;
   }
   tmp->_next = stmt;
 }
 
-const ResourceIDs
+ResourceIDs
 Statement::get_resource_ids() const
 {
   const Statement *stmt = this;
@@ -69,6 +69,8 @@ Statement::initialize_hooks()
   add_allowed_hook(TS_HTTP_SEND_REQUEST_HDR_HOOK);
   add_allowed_hook(TS_HTTP_SEND_RESPONSE_HDR_HOOK);
   add_allowed_hook(TS_REMAP_PSEUDO_HOOK);
+  add_allowed_hook(TS_HTTP_TXN_START_HOOK);
+  add_allowed_hook(TS_HTTP_TXN_CLOSE_HOOK);
 }
 
 // Parse URL qualifiers, this one is special since it's used in a few places.
