@@ -35,7 +35,7 @@
 #define MAX_SIMPLE_RETRIES 5
 #define MAX_UNAVAILABLE_SERVER_RETRIES 5
 
-typedef ControlMatcher<ParentRecord, ParentResult> P_table;
+using P_table = ControlMatcher<ParentRecord, ParentResult>;
 
 // Global Vars for Parent Selection
 static const char modulePrefix[]                             = "[ParentSelection]";
@@ -300,10 +300,6 @@ ParentConfig::reconfigure()
   Note("%s finished loading", ts::filename::PARENT);
 }
 
-// void ParentConfig::print
-//
-//   Debugging function
-//
 void
 ParentConfig::print()
 {
@@ -394,10 +390,10 @@ ParentRecord::PreProcessParents(const char *val, const int line_num, char *buf,
   token = strtok_r(_val, PARENT_DELIMITERS, &savePtr);
   while (token != nullptr) {
     if ((nm = strchr(token, ':')) != nullptr) {
-      size_t len = (nm - token);
-      ink_assert(len < sizeof(fqdn));
+      size_t length = (nm - token);
+      ink_assert(length < sizeof(fqdn));
       memset(fqdn, 0, sizeof(fqdn));
-      strncpy(fqdn, token, len);
+      strncpy(fqdn, token, length);
       if (self_detect && machine->is_self(fqdn)) {
         if (self_detect == 1) {
           Debug("parent_select", "token: %s, matches this machine.  Removing self from parent list at line %d", fqdn, line_num);
@@ -866,7 +862,7 @@ ParentRecord::~ParentRecord()
 }
 
 void
-ParentRecord::Print()
+ParentRecord::Print() const
 {
   printf("\t\t");
   for (int i = 0; i < num_parents; i++) {
