@@ -26,15 +26,12 @@
 #include <cassert>
 #include "tscore/Arena.h"
 #include "tscore/CryptoHash.h"
+#include "tscore/HTTPVersion.h"
 #include "MIME.h"
 #include "URL.h"
 
 #include "tscore/ink_apidefs.h"
 
-#define HTTP_VERSION(a, b) ((((a)&0xFFFF) << 16) | ((b)&0xFFFF))
-#define HTTP_MINOR(v) ((v)&0xFFFF)
-#define HTTP_MAJOR(v) (((v) >> 16) & 0xFFFF)
-
 class Http2HeaderTable;
 
 enum HTTPStatus {
@@ -247,8 +244,8 @@ enum HTTPType {
 
 struct HTTPHdrImpl : public HdrHeapObjImpl {
   // HdrHeapObjImpl is 4 bytes
-  HTTPType m_polarity; // request or response or unknown
-  int32_t m_version;   // cooked version number
+  HTTPType m_polarity;   // request or response or unknown
+  HTTPVersion m_version; // cooked version number
   // 12 bytes means 4 bytes padding here on 64-bit architectures
   union {
     struct {
@@ -425,8 +422,7 @@ inkcoreapi int http_hdr_print(HdrHeap *heap, HTTPHdrImpl *hh, char *buf, int buf
 
 void http_hdr_describe(HdrHeapObjImpl *obj, bool recurse = true);
 
-// int32_t                  http_hdr_version_get (HTTPHdrImpl *hh);
-inkcoreapi void http_hdr_version_set(HTTPHdrImpl *hh, int32_t ver);
+inkcoreapi bool http_hdr_version_set(HTTPHdrImpl *hh, const HTTPVersion &ver);
 
 const char *http_hdr_method_get(HTTPHdrImpl *hh, int *length);
 inkcoreapi void http_hdr_method_set(HdrHeap *heap, HTTPHdrImpl *hh, const char *method, int16_t method_wks_idx, int method_length,
@@ -443,15 +439,16 @@ const char *http_hdr_reason_lookup(unsigned status);
 void http_parser_init(HTTPParser *parser);
 void http_parser_clear(HTTPParser *parser);
 ParseResult http_parser_parse_req(HTTPParser *parser, HdrHeap *heap, HTTPHdrImpl *hh, const char **start, const char *end,
-                                  bool must_copy_strings, bool eof, bool strict_uri_parsing, size_t max_request_line_size,
+                                  bool must_copy_strings, bool eof, int strict_uri_parsing, size_t max_request_line_size,
                                   size_t max_hdr_field_size);
+ParseResult validate_hdr_request_target(int method_wks_idx, URLImpl *url);
 ParseResult validate_hdr_host(HTTPHdrImpl *hh);
 ParseResult validate_hdr_content_length(HdrHeap *heap, HTTPHdrImpl *hh);
 ParseResult http_parser_parse_resp(HTTPParser *parser, HdrHeap *heap, HTTPHdrImpl *hh, const char **start, const char *end,
                                    bool must_copy_strings, bool eof);
 
 HTTPStatus http_parse_status(const char *start, const char *end);
-int32_t http_parse_version(const char *start, const char *end);
+HTTPVersion http_parse_version(const char *start, const char *end);
 
 /*
 HTTPValAccept*         http_parse_accept (const char *buf, Arena *arena);
@@ -464,28 +461,7 @@ HTTPValRange*          http_parse_range (const char *buf, Arena *arena);
 */
 HTTPValTE *http_parse_te(const char *buf, int len, Arena *arena);
 
-class HTTPVersion
-{
-public:
-  HTTPVersion()                        = default;
-  HTTPVersion(HTTPVersion const &that) = default;
-  explicit HTTPVersion(int32_t version);
-  HTTPVersion(int ver_major, int ver_minor);
-
-  void set(HTTPVersion ver);
-  void set(int ver_major, int ver_minor);
-
-  HTTPVersion &operator=(const HTTPVersion &hv) = default;
-  int operator==(const HTTPVersion &hv) const;
-  int operator!=(const HTTPVersion &hv) const;
-  int operator>(const HTTPVersion &hv) const;
-  int operator<(const HTTPVersion &hv) const;
-  int operator>=(const HTTPVersion &hv) const;
-  int operator<=(const HTTPVersion &hv) const;
-
-public:
-  int32_t m_version{HTTP_VERSION(1, 0)};
-};
+bool is_http1_hdr_version_supported(const HTTPVersion &http_version);
 
 class IOBufferReader;
 
@@ -546,9 +522,8 @@ public:
       and invoking @c URL::string_get if the host is in a header
       field and not explicitly in the URL.
    */
-  char *url_string_get(Arena *arena    = nullptr, ///< Arena to use, or @c malloc if NULL.
-                       int *length     = nullptr, ///< Store string length here.
-                       bool normalized = false    ///< Scheme and host normalized to lower case letters.
+  char *url_string_get(Arena *arena = nullptr, ///< Arena to use, or @c malloc if NULL.
+                       int *length  = nullptr  ///< Store string length here.
   );
   /** Get a string with the effective URL in it.
       This is automatically allocated if needed in the request heap.
@@ -562,17 +537,17 @@ public:
       Output is not null terminated.
       @return 0 on failure, non-zero on success.
    */
-  int url_print(char *buff,             ///< Output buffer
-                int length,             ///< Length of @a buffer
-                int *offset,            ///< [in,out] ???
-                int *skip,              ///< [in,out] ???
-                bool normalized = false ///< host/scheme normalized to lower case
+  int url_print(char *buff,                                       ///< Output buffer
+                int length,                                       ///< Length of @a buffer
+                int *offset,                                      ///< [in,out] ???
+                int *skip,                                        ///< [in,out] ???
+                unsigned normalization_flags = URLNormalize::NONE ///< host/scheme normalized to lower case
   );
 
   /** Return the length of the URL that url_print() will create.
       @return -1 on failure, non-negative on success.
    */
-  int url_printed_length();
+  int url_printed_length(unsigned normalizaion_flags = URLNormalize::NONE);
 
   /** Get the URL path.
       This is a reference, not allocated.
@@ -581,6 +556,27 @@ public:
   const char *path_get(int *length ///< Storage for path length.
   );
 
+  /** Get the URL matrix params.
+      This is a reference, not allocated.
+      @return A pointer to the matrix params or @c NULL if there is no valid URL.
+  */
+  const char *params_get(int *length ///< Storage for param length.
+  );
+
+  /** Get the URL query.
+      This is a reference, not allocated.
+      @return A pointer to the query or @c NULL if there is no valid URL.
+  */
+  const char *query_get(int *length ///< Storage for query length.
+  );
+
+  /** Get the URL fragment.
+      This is a reference, not allocated.
+      @return A pointer to the fragment or @c NULL if there is no valid URL.
+  */
+  const char *fragment_get(int *length ///< Storage for fragment length.
+  );
+
   /** Get the target host name.
       The length is returned in @a length if non-NULL.
       @note The results are cached so this is fast after the first call.
@@ -635,11 +631,11 @@ public:
   void mark_early_data(bool flag = true) const;
   bool is_early_data() const;
 
-  ParseResult parse_req(HTTPParser *parser, const char **start, const char *end, bool eof, bool strict_uri_parsing = false,
+  ParseResult parse_req(HTTPParser *parser, const char **start, const char *end, bool eof, int strict_uri_parsing = 0,
                         size_t max_request_line_size = UINT16_MAX, size_t max_hdr_field_size = 131070);
   ParseResult parse_resp(HTTPParser *parser, const char **start, const char *end, bool eof);
 
-  ParseResult parse_req(HTTPParser *parser, IOBufferReader *r, int *bytes_used, bool eof, bool strict_uri_parsing = false,
+  ParseResult parse_req(HTTPParser *parser, IOBufferReader *r, int *bytes_used, bool eof, int strict_uri_parsing = 0,
                         size_t max_request_line_size = UINT16_MAX, size_t max_hdr_field_size = UINT16_MAX);
   ParseResult parse_resp(HTTPParser *parser, IOBufferReader *r, int *bytes_used, bool eof);
 
@@ -675,88 +671,6 @@ private:
   friend class UrlPrintHack; // don't ask.
 };
 
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline HTTPVersion::HTTPVersion(int32_t version) : m_version(version) {}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline HTTPVersion::HTTPVersion(int ver_major, int ver_minor) : m_version(HTTP_VERSION(ver_major, ver_minor)) {}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline void
-HTTPVersion::set(HTTPVersion ver)
-{
-  m_version = ver.m_version;
-}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline void
-HTTPVersion::set(int ver_major, int ver_minor)
-{
-  m_version = HTTP_VERSION(ver_major, ver_minor);
-}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline int
-HTTPVersion::operator==(const HTTPVersion &hv) const
-{
-  return (m_version == hv.m_version);
-}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline int
-HTTPVersion::operator!=(const HTTPVersion &hv) const
-{
-  return (m_version != hv.m_version);
-}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline int
-HTTPVersion::operator>(const HTTPVersion &hv) const
-{
-  return (m_version > hv.m_version);
-}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline int
-HTTPVersion::operator<(const HTTPVersion &hv) const
-{
-  return (m_version < hv.m_version);
-}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline int
-HTTPVersion::operator>=(const HTTPVersion &hv) const
-{
-  return (m_version >= hv.m_version);
-}
-
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline int
-HTTPVersion::operator<=(const HTTPVersion &hv) const
-{
-  return (m_version <= hv.m_version);
-}
-
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
@@ -932,15 +846,6 @@ HTTPHdr::type_get() const
   return http_hdr_type_get(m_http);
 }
 
-/*-------------------------------------------------------------------------
-  -------------------------------------------------------------------------*/
-
-inline int32_t
-http_hdr_version_get(HTTPHdrImpl *hh)
-{
-  return (hh->m_version);
-}
-
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
@@ -948,7 +853,7 @@ inline HTTPVersion
 HTTPHdr::version_get() const
 {
   ink_assert(valid());
-  return HTTPVersion(http_hdr_version_get(m_http));
+  return m_http->m_version;
 }
 
 /*-------------------------------------------------------------------------
@@ -974,14 +879,14 @@ is_header_keep_alive(const HTTPVersion &http_version, const MIMEField *con_hdr)
       con_token = CON_TOKEN_CLOSE;
   }
 
-  if (HTTPVersion(1, 0) == http_version) {
+  if (HTTP_1_0 == http_version) {
     keep_alive = (con_token == CON_TOKEN_KEEP_ALIVE) ? (HTTP_KEEPALIVE) : (HTTP_NO_KEEPALIVE);
-  } else if (HTTPVersion(1, 1) == http_version) {
+  } else if (HTTP_1_1 == http_version) {
     // We deviate from the spec here.  If the we got a response where
     //   where there is no Connection header and the request 1.0 was
     //   1.0 don't treat this as keep-alive since Netscape-Enterprise/3.6 SP1
     //   server doesn't
-    keep_alive = ((con_token == CON_TOKEN_KEEP_ALIVE) || (con_token == CON_TOKEN_NONE && HTTPVersion(1, 1) == http_version)) ?
+    keep_alive = ((con_token == CON_TOKEN_KEEP_ALIVE) || (con_token == CON_TOKEN_NONE && HTTP_1_1 == http_version)) ?
                    (HTTP_KEEPALIVE) :
                    (HTTP_NO_KEEPALIVE);
   } else {
@@ -1035,7 +940,7 @@ inline void
 HTTPHdr::version_set(HTTPVersion version)
 {
   ink_assert(valid());
-  http_hdr_version_set(m_http, version.m_version);
+  http_hdr_version_set(m_http, version);
 }
 
 /*-------------------------------------------------------------------------
@@ -1250,7 +1155,7 @@ HTTPHdr::is_early_data() const
   -------------------------------------------------------------------------*/
 
 inline ParseResult
-HTTPHdr::parse_req(HTTPParser *parser, const char **start, const char *end, bool eof, bool strict_uri_parsing,
+HTTPHdr::parse_req(HTTPParser *parser, const char **start, const char *end, bool eof, int strict_uri_parsing,
                    size_t max_request_line_size, size_t max_hdr_field_size)
 {
   ink_assert(valid());
@@ -1314,6 +1219,27 @@ HTTPHdr::path_get(int *length)
   return url ? url->path_get(length) : nullptr;
 }
 
+inline const char *
+HTTPHdr::params_get(int *length)
+{
+  URL *url = this->url_get();
+  return url ? url->params_get(length) : nullptr;
+}
+
+inline const char *
+HTTPHdr::query_get(int *length)
+{
+  URL *url = this->url_get();
+  return url ? url->query_get(length) : nullptr;
+}
+
+inline const char *
+HTTPHdr::fragment_get(int *length)
+{
+  URL *url = this->url_get();
+  return url ? url->fragment_get(length) : nullptr;
+}
+
 inline const char *
 HTTPHdr::scheme_get(int *length)
 {
