@@ -114,6 +114,36 @@ validate_host_name(std::string_view addr)
   return std::all_of(addr.begin(), addr.end(), &is_host_char);
 }
 
+/**
+   Checks if the (un-well-known) scheme is valid
+
+   RFC 3986 Section 3.1
+   These are the valid characters in a scheme:
+     scheme      = ALPHA *( ALPHA / DIGIT / "+" / "-" / "." )
+   return an error if there is another character in the scheme
+*/
+bool
+validate_scheme(std::string_view scheme)
+{
+  if (scheme.empty()) {
+    return false;
+  }
+
+  if (!ParseRules::is_alpha(scheme[0])) {
+    return false;
+  }
+
+  for (size_t i = 0; i < scheme.size(); ++i) {
+    const char &c = scheme[i];
+
+    if (!(ParseRules::is_alnum(c) != 0 || c == '+' || c == '-' || c == '.')) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
@@ -612,40 +642,41 @@ url_clear_string_ref(URLImpl *url)
 }
 
 char *
-url_string_get_ref(HdrHeap *heap, URLImpl *url, int *length, bool normalized)
+url_string_get_ref(HdrHeap *heap, URLImpl *url, int *length, unsigned normalization_flags)
 {
   if (!url) {
     return nullptr;
   }
 
-  if (url->m_ptr_printed_string && url->m_clean) {
+  if (url->m_ptr_printed_string && url->m_clean && (normalization_flags == url->m_normalization_flags)) {
     if (length) {
       *length = url->m_len_printed_string;
     }
     return const_cast<char *>(url->m_ptr_printed_string);
   } else { // either not clean or never printed
-    int len = url_length_get(url);
+    int len = url_length_get(url, normalization_flags);
     char *buf;
     int index  = 0;
     int offset = 0;
 
     /* stuff alloc'd here gets gc'd on HdrHeap::destroy() */
     buf = heap->allocate_str(len + 1);
-    url_print(url, buf, len, &index, &offset, normalized);
+    url_print(url, buf, len, &index, &offset, normalization_flags);
     buf[len] = '\0';
 
     if (length) {
       *length = len;
     }
-    url->m_clean              = true; // reset since we have url_print()'ed again
-    url->m_len_printed_string = len;
-    url->m_ptr_printed_string = buf;
+    url->m_clean               = true; // reset since we have url_print()'ed again
+    url->m_len_printed_string  = len;
+    url->m_ptr_printed_string  = buf;
+    url->m_normalization_flags = normalization_flags;
     return buf;
   }
 }
 
 char *
-url_string_get(URLImpl *url, Arena *arena, int *length, HdrHeap *heap, bool normalized)
+url_string_get(URLImpl *url, Arena *arena, int *length, HdrHeap *heap)
 {
   int len = url_length_get(url);
   char *buf;
@@ -655,7 +686,7 @@ url_string_get(URLImpl *url, Arena *arena, int *length, HdrHeap *heap, bool norm
 
   buf = arena ? arena->str_alloc(len) : static_cast<char *>(ats_malloc(len + 1));
 
-  url_print(url, buf, len, &index, &offset, normalized);
+  url_print(url, buf, len, &index, &offset);
   buf[len] = '\0';
 
   /* see string_get_ref() */
@@ -801,15 +832,19 @@ url_type_get(URLImpl *url)
   -------------------------------------------------------------------------*/
 
 int
-url_length_get(URLImpl *url)
+url_length_get(URLImpl *url, unsigned normalization_flags)
 {
   int length = 0;
 
   if (url->m_ptr_scheme) {
-    if ((url->m_scheme_wks_idx >= 0) && (hdrtoken_index_to_wks(url->m_scheme_wks_idx) == URL_SCHEME_FILE)) {
-      length += url->m_len_scheme + 1; // +1 for ":"
-    } else {
-      length += url->m_len_scheme + 3; // +3 for "://"
+    length += url->m_len_scheme + 3; // +3 for "://"
+
+  } else if (normalization_flags & URLNormalize::IMPLIED_SCHEME) {
+    if (URL_TYPE_HTTP == url->m_url_type) {
+      length += URL_LEN_HTTP + 3;
+
+    } else if (URL_TYPE_HTTPS == url->m_url_type) {
+      length += URL_LEN_HTTPS + 3;
     }
   }
 
@@ -1142,19 +1177,9 @@ url_parse_scheme(HdrHeap *heap, URLImpl *url, const char **start, const char *en
 
         if (!(scheme_wks_idx > 0 && hdrtoken_wks_to_token_type(scheme_wks) == HDRTOKEN_TYPE_SCHEME)) {
           // Unknown scheme, validate the scheme
-
-          // RFC 3986 Section 3.1
-          // These are the valid characters in a scheme:
-          //   scheme      = ALPHA *( ALPHA / DIGIT / "+" / "-" / "." )
-          // return an error if there is another character in the scheme
-          if (!ParseRules::is_alpha(*scheme_start)) {
+          if (!validate_scheme({scheme_start, static_cast<size_t>(scheme_end - scheme_start)})) {
             return PARSE_RESULT_ERROR;
           }
-          for (cur = scheme_start + 1; cur < scheme_end; ++cur) {
-            if (!(ParseRules::is_alnum(*cur) != 0 || *cur == '+' || *cur == '-' || *cur == '.')) {
-              return PARSE_RESULT_ERROR;
-            }
-          }
         }
         url_scheme_set(heap, url, scheme_start, scheme_wks_idx, scheme_end - scheme_start, copy_strings_p);
       }
@@ -1186,14 +1211,38 @@ url_is_strictly_compliant(const char *start, const char *end)
   return true;
 }
 
+/**
+ *  This method will return TRUE if the uri is mostly compliant with
+ *  RFC 3986 and it will return FALSE if not. Specifically denying white
+ *  space an unprintable characters
+ */
+bool
+url_is_mostly_compliant(const char *start, const char *end)
+{
+  for (const char *i = start; i < end; ++i) {
+    if (isspace(*i)) {
+      Debug("http", "Whitespace character [0x%.2X] found in URL", (unsigned char)*i);
+      return false;
+    }
+    if (!isprint(*i)) {
+      Debug("http", "Non-printable character [0x%.2X] found in URL", (unsigned char)*i);
+      return false;
+    }
+  }
+  return true;
+}
+
 } // namespace UrlImpl
 using namespace UrlImpl;
 
 ParseResult
-url_parse(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings_p, bool strict_uri_parsing,
+url_parse(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings_p, int strict_uri_parsing,
           bool verify_host_characters)
 {
-  if (strict_uri_parsing && !url_is_strictly_compliant(*start, end)) {
+  if (strict_uri_parsing == 1 && !url_is_strictly_compliant(*start, end)) {
+    return PARSE_RESULT_ERROR;
+  }
+  if (strict_uri_parsing == 2 && !url_is_mostly_compliant(*start, end)) {
     return PARSE_RESULT_ERROR;
   }
 
@@ -1582,15 +1631,30 @@ url_parse_http_regex(HdrHeap *heap, URLImpl *url, const char **start, const char
  ***********************************************************************/
 
 int
-url_print(URLImpl *url, char *buf_start, int buf_length, int *buf_index_inout, int *buf_chars_to_skip_inout, bool normalize)
+url_print(URLImpl *url, char *buf_start, int buf_length, int *buf_index_inout, int *buf_chars_to_skip_inout,
+          unsigned normalization_flags)
 {
 #define TRY(x) \
   if (!x)      \
   return 0
 
+  bool scheme_added = false;
   if (url->m_ptr_scheme) {
-    TRY((normalize ? mime_mem_print_lc : mime_mem_print)(url->m_ptr_scheme, url->m_len_scheme, buf_start, buf_length,
-                                                         buf_index_inout, buf_chars_to_skip_inout));
+    TRY(((normalization_flags & URLNormalize::LC_SCHEME_HOST) ? mime_mem_print_lc : mime_mem_print)(
+      url->m_ptr_scheme, url->m_len_scheme, buf_start, buf_length, buf_index_inout, buf_chars_to_skip_inout));
+    scheme_added = true;
+
+  } else if (normalization_flags & URLNormalize::IMPLIED_SCHEME) {
+    if (URL_TYPE_HTTP == url->m_url_type) {
+      TRY(mime_mem_print(URL_SCHEME_HTTP, URL_LEN_HTTP, buf_start, buf_length, buf_index_inout, buf_chars_to_skip_inout));
+      scheme_added = true;
+
+    } else if (URL_TYPE_HTTPS == url->m_url_type) {
+      TRY(mime_mem_print(URL_SCHEME_HTTPS, URL_LEN_HTTPS, buf_start, buf_length, buf_index_inout, buf_chars_to_skip_inout));
+      scheme_added = true;
+    }
+  }
+  if (scheme_added) {
     TRY(mime_mem_print("://", 3, buf_start, buf_length, buf_index_inout, buf_chars_to_skip_inout));
   }
 
@@ -1612,8 +1676,8 @@ url_print(URLImpl *url, char *buf_start, int buf_length, int *buf_index_inout, i
     if (bracket_p) {
       TRY(mime_mem_print("[", 1, buf_start, buf_length, buf_index_inout, buf_chars_to_skip_inout));
     }
-    TRY((normalize ? mime_mem_print_lc : mime_mem_print)(url->m_ptr_host, url->m_len_host, buf_start, buf_length, buf_index_inout,
-                                                         buf_chars_to_skip_inout));
+    TRY(((normalization_flags & URLNormalize::LC_SCHEME_HOST) ? mime_mem_print_lc : mime_mem_print)(
+      url->m_ptr_host, url->m_len_host, buf_start, buf_length, buf_index_inout, buf_chars_to_skip_inout));
     if (bracket_p) {
       TRY(mime_mem_print("]", 1, buf_start, buf_length, buf_index_inout, buf_chars_to_skip_inout));
     }
