@@ -76,7 +76,8 @@ struct URLImpl : public HdrHeapObjImpl {
   uint32_t m_clean : 1;
   /// Whether the URI had an absolutely empty path, not even an initial '/'.
   uint32_t m_path_is_empty : 1;
-  // 8 bytes + 2 bits, will result in padding
+  uint32_t m_normalization_flags : 2; // Only valid if both m_clean and m_ptr_printed_sting are non-zero.
+  // 8 bytes + 4 bits, will result in padding
 
   // Marshaling Functions
   int marshal(MarshalXlate *str_xlate, int num_xlate);
@@ -156,6 +157,8 @@ extern int URL_LEN_MMST;
 
 /* Public */
 bool validate_host_name(std::string_view addr);
+bool validate_scheme(std::string_view scheme);
+
 void url_init();
 
 URLImpl *url_create(HdrHeap *heap);
@@ -166,13 +169,22 @@ URLImpl *url_copy(URLImpl *s_url, HdrHeap *s_heap, HdrHeap *d_heap, bool inherit
 void url_copy_onto(URLImpl *s_url, HdrHeap *s_heap, URLImpl *d_url, HdrHeap *d_heap, bool inherit_strs = true);
 void url_copy_onto_as_server_url(URLImpl *s_url, HdrHeap *s_heap, URLImpl *d_url, HdrHeap *d_heap, bool inherit_strs = true);
 
-int url_print(URLImpl *u, char *buf, int bufsize, int *bufindex, int *dumpoffset, bool normalized = false);
+// Normalization flag masks.
+namespace URLNormalize
+{
+unsigned const NONE           = 0;
+unsigned const IMPLIED_SCHEME = 1; // If scheme missing, add scheme implied by URL type.
+unsigned const LC_SCHEME_HOST = 2; // Force scheme and host to lower case if necessary.
+};                                 // namespace URLNormalize
+
+int url_print(URLImpl *u, char *buf, int bufsize, int *bufindex, int *dumpoffset,
+              unsigned normalization_flags = URLNormalize::NONE);
 void url_describe(HdrHeapObjImpl *raw, bool recurse);
 
-int url_length_get(URLImpl *url);
-char *url_string_get(URLImpl *url, Arena *arena, int *length, HdrHeap *heap, bool normalized = false);
+int url_length_get(URLImpl *url, unsigned normalization_flags = URLNormalize::NONE);
+char *url_string_get(URLImpl *url, Arena *arena, int *length, HdrHeap *heap);
 void url_clear_string_ref(URLImpl *url);
-char *url_string_get_ref(HdrHeap *heap, URLImpl *url, int *length, bool normalized = false);
+char *url_string_get_ref(HdrHeap *heap, URLImpl *url, int *length, unsigned normalization_flags = URLNormalize::NONE);
 void url_called_set(URLImpl *url);
 char *url_string_get_buf(URLImpl *url, char *dstbuf, int dstbuf_size, int *length);
 
@@ -199,7 +211,7 @@ void url_fragment_set(HdrHeap *heap, URLImpl *url, const char *value, int length
 constexpr bool USE_STRICT_URI_PARSING = true;
 
 ParseResult url_parse(HdrHeap *heap, URLImpl *url, const char **start, const char *end, bool copy_strings,
-                      bool strict_uri_parsing = false, bool verify_host_characters = true);
+                      int strict_uri_parsing = false, bool verify_host_characters = true);
 
 constexpr bool COPY_STRINGS = true;
 
@@ -245,19 +257,21 @@ public:
   // Note that URL::destroy() is inherited from HdrHeapSDKHandle.
   void nuke_proxy_stuff();
 
-  int print(char *buf, int bufsize, int *bufindex, int *dumpoffset, bool normalized = false);
+  int print(char *buf, int bufsize, int *bufindex, int *dumpoffset, unsigned normalization_flags = URLNormalize::NONE) const;
+
+  int length_get(unsigned normalization_flags = URLNormalize::NONE) const;
 
-  int length_get();
   void clear_string_ref();
-  char *string_get(Arena *arena, int *length = nullptr, bool normalized = false);
-  char *string_get_ref(int *length = nullptr, bool normalized = false);
-  char *string_get_buf(char *dstbuf, int dsbuf_size, int *length = nullptr);
+
+  char *string_get(Arena *arena, int *length = nullptr) const;
+  char *string_get_ref(int *length = nullptr, unsigned normalization_flags = URLNormalize::NONE) const;
+  char *string_get_buf(char *dstbuf, int dsbuf_size, int *length = nullptr) const;
   void hash_get(CryptoHash *hash, cache_generation_t generation = -1) const;
-  void host_hash_get(CryptoHash *hash);
+  void host_hash_get(CryptoHash *hash) const;
 
   const char *scheme_get(int *length);
   const std::string_view scheme_get();
-  int scheme_get_wksidx();
+  int scheme_get_wksidx() const;
   void scheme_set(const char *value, int length);
 
   const char *user_get(int *length);
@@ -266,8 +280,9 @@ public:
   void password_set(const char *value, int length);
   const char *host_get(int *length);
   void host_set(const char *value, int length);
-  int port_get();
-  int port_get_raw();
+
+  int port_get() const;
+  int port_get_raw() const;
   void port_set(int port);
 
   const char *path_get(int *length);
@@ -412,37 +427,37 @@ URL::nuke_proxy_stuff()
   -------------------------------------------------------------------------*/
 
 inline int
-URL::print(char *buf, int bufsize, int *bufindex, int *dumpoffset, bool normalized)
+URL::print(char *buf, int bufsize, int *bufindex, int *dumpoffset, unsigned normalization_flags) const
 {
   ink_assert(valid());
-  return url_print(m_url_impl, buf, bufsize, bufindex, dumpoffset, normalized);
+  return url_print(m_url_impl, buf, bufsize, bufindex, dumpoffset, normalization_flags);
 }
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
 inline int
-URL::length_get()
+URL::length_get(unsigned normalization_flags) const
 {
   ink_assert(valid());
-  return url_length_get(m_url_impl);
+  return url_length_get(m_url_impl, normalization_flags);
 }
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
 inline char *
-URL::string_get(Arena *arena_or_null_for_malloc, int *length, bool normalized)
+URL::string_get(Arena *arena_or_null_for_malloc, int *length) const
 {
   ink_assert(valid());
-  return url_string_get(m_url_impl, arena_or_null_for_malloc, length, m_heap, normalized);
+  return url_string_get(m_url_impl, arena_or_null_for_malloc, length, m_heap);
 }
 
 inline char *
-URL::string_get_ref(int *length, bool normalized)
+URL::string_get_ref(int *length, unsigned normalization_flags) const
 {
   ink_assert(valid());
-  return url_string_get_ref(m_heap, m_url_impl, length, normalized);
+  return url_string_get_ref(m_heap, m_url_impl, length, normalization_flags);
 }
 
 inline void
@@ -456,7 +471,7 @@ URL::clear_string_ref()
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 inline char *
-URL::string_get_buf(char *dstbuf, int dsbuf_size, int *length)
+URL::string_get_buf(char *dstbuf, int dsbuf_size, int *length) const
 {
   ink_assert(valid());
   return url_string_get_buf(m_url_impl, dstbuf, dsbuf_size, length);
@@ -476,7 +491,7 @@ URL::hash_get(CryptoHash *hash, cache_generation_t generation) const
   -------------------------------------------------------------------------*/
 
 inline void
-URL::host_hash_get(CryptoHash *hash)
+URL::host_hash_get(CryptoHash *hash) const
 {
   ink_assert(valid());
   url_host_CryptoHash_get(m_url_impl, hash);
@@ -507,7 +522,7 @@ URL::scheme_get(int *length)
 }
 
 inline int
-URL::scheme_get_wksidx()
+URL::scheme_get_wksidx() const
 {
   ink_assert(valid());
   return (m_url_impl->m_scheme_wks_idx);
@@ -591,7 +606,7 @@ URL::host_set(const char *value, int length)
   -------------------------------------------------------------------------*/
 
 inline int
-URL::port_get()
+URL::port_get() const
 {
   ink_assert(valid());
   return url_canonicalize_port(m_url_impl->m_url_type, m_url_impl->m_port);
@@ -601,7 +616,7 @@ URL::port_get()
   -------------------------------------------------------------------------*/
 
 inline int
-URL::port_get_raw()
+URL::port_get_raw() const
 {
   ink_assert(valid());
   return m_url_impl->m_port;
