@@ -167,6 +167,54 @@ enum HttpPluginTunnel_t {
 class CoreUtils;
 class PluginVCCore;
 
+class PendingAction
+{
+public:
+  bool
+  is_empty() const
+  {
+    return pending_action == nullptr;
+  }
+  PendingAction &
+  operator=(Action *b)
+  {
+    // Don't do anything if the new action is _DONE
+    if (b != ACTION_RESULT_DONE) {
+      if (b != pending_action && pending_action != nullptr) {
+        pending_action->cancel();
+      }
+      pending_action = b;
+    }
+    return *this;
+  }
+  Continuation *
+  get_continuation() const
+  {
+    return pending_action ? pending_action->continuation : nullptr;
+  }
+  Action *
+  get() const
+  {
+    return pending_action;
+  }
+  void
+  clear_if_action_is(Action *current_action)
+  {
+    if (current_action == pending_action) {
+      pending_action = nullptr;
+    }
+  }
+  ~PendingAction()
+  {
+    if (pending_action) {
+      pending_action->cancel();
+    }
+  }
+
+private:
+  Action *pending_action = nullptr;
+};
+
 class PostDataBuffers
 {
 public:
@@ -210,9 +258,7 @@ public:
   virtual void destroy();
 
   static HttpSM *allocate();
-  HttpCacheSM &get_cache_sm();          // Added to get the object of CacheSM YTS Team, yamsat
-  HttpVCTableEntry *get_ua_entry();     // Added to get the ua_entry pointer  - YTS-TEAM
-  HttpVCTableEntry *get_server_entry(); // Added to get the server_entry pointer
+  HttpCacheSM &get_cache_sm(); // Added to get the object of CacheSM YTS Team, yamsat
   std::string_view get_outbound_sni() const;
   std::string_view get_outbound_cert() const;
 
@@ -223,15 +269,11 @@ public:
   // Called by httpSessionManager so that we can reset
   //  the session timeouts and initiate a read while
   //  holding the lock for the server session
-  void attach_server_session(Http1ServerSession *s);
+  void attach_server_session(PoolableSession *s);
 
   // Used to read attributes of
   // the current active server session
-  Http1ServerSession *
-  get_server_session()
-  {
-    return server_session;
-  }
+  PoolableSession *get_server_session() const;
 
   ProxyTransaction *
   get_ua_txn()
@@ -244,6 +286,11 @@ public:
   //  directly from transact
   void do_hostdb_update_if_necessary();
 
+  // Look at the configured policy and the current server connect_result
+  // to deterine whether this connection attempt should contribute to the
+  // dead server count
+  bool track_connect_fail() const;
+
   // Called by transact. Decide if cached response supports Range and
   // setup Range transfomration if so.
   // return true when the Range is unsatisfiable
@@ -305,6 +352,13 @@ public:
   /// @arg n [in] Size of the array @a result.
   int populate_client_protocol(std::string_view *result, int n) const;
   const char *client_protocol_contains(std::string_view tag_prefix) const;
+
+  /// Get the protocol stack for the outbound (origin server) connection.
+  /// @arg result [out] Array to store the results
+  /// @arg n [in] Size of the array @a result.
+  int populate_server_protocol(std::string_view *result, int n) const;
+  const char *server_protocol_contains(std::string_view tag_prefix) const;
+
   std::string_view find_proto_string(HTTPVersion version) const;
 
   int64_t sm_id      = -1;
@@ -353,7 +407,6 @@ protected:
   HttpVCTable vc_table;
 
   HttpVCTableEntry *ua_entry = nullptr;
-  void remove_ua_entry();
 
 public:
   ProxyTransaction *ua_txn         = nullptr;
@@ -376,9 +429,7 @@ protected:
    * we should create a new connection and then once we attach the session we'll mark it as private.
    */
   bool will_be_private_ss              = false;
-  int shared_session_retries           = 0;
   IOBufferReader *server_buffer_reader = nullptr;
-  void remove_server_entry();
 
   HttpTransformInfo transform_info;
   HttpTransformInfo post_transform_info;
@@ -390,8 +441,8 @@ protected:
   HttpCacheSM transform_cache_sm;
 
   HttpSMHandler default_handler = nullptr;
-  Action *pending_action        = nullptr;
-  Continuation *schedule_cont   = nullptr;
+  PendingAction pending_action;
+  Continuation *schedule_cont = nullptr;
 
   HTTPParser http_parser;
   void start_sub_sm();
@@ -479,7 +530,6 @@ protected:
   void handle_http_server_open();
   void handle_post_failure();
   void mark_host_failure(HostDBInfo *info, time_t time_down);
-  void mark_server_down_on_client_abort();
   void release_server_session(bool serve_from_cache = false);
   void set_ua_abort(HttpTransact::AbortState_t ua_abort, int event);
   int write_header_into_buffer(HTTPHdr *h, MIOBuffer *b);
@@ -549,9 +599,11 @@ public:
   bool is_using_post_buffer           = false;
   std::optional<bool> mptcp_state; // Don't initialize, that marks it as "not defined".
   const char *client_protocol     = "-";
+  const char *server_protocol     = "-";
   const char *client_sec_protocol = "-";
   const char *client_cipher_suite = "-";
   const char *client_curve        = "-";
+  int client_alpn_id              = SessionProtocolNameRegistry::INVALID;
   int server_transact_count       = 0;
 
   TransactionMilestones milestones;
@@ -590,7 +642,6 @@ protected:
   bool terminate_sm         = false;
   bool kill_this_async_done = false;
   bool parse_range_done     = false;
-  virtual int kill_this_async_hook(int event, void *data);
   void kill_this();
   void update_stats();
   void transform_cleanup(TSHttpHookID hook, HttpTransformInfo *info);
@@ -632,9 +683,9 @@ public:
     return _client_transaction_priority_dependence;
   }
 
-  void set_server_netvc_inactivity_timeout(NetVConnection *netvc);
-  void set_server_netvc_active_timeout(NetVConnection *netvc);
-  void set_server_netvc_connect_timeout(NetVConnection *netvc);
+  ink_hrtime get_server_inactivity_timeout();
+  ink_hrtime get_server_active_timeout();
+  ink_hrtime get_server_connect_timeout();
   void rewind_state_machine();
 
 private:
@@ -651,42 +702,6 @@ HttpSM::get_cache_sm()
   return cache_sm;
 }
 
-// Function to get the ua_entry pointer - YTS Team, yamsat
-inline HttpVCTableEntry *
-HttpSM::get_ua_entry()
-{
-  return ua_entry;
-}
-
-inline HttpVCTableEntry *
-HttpSM::get_server_entry()
-{
-  return server_entry;
-}
-
-inline HttpSM *
-HttpSM::allocate()
-{
-  extern ClassAllocator<HttpSM> httpSMAllocator;
-  return httpSMAllocator.alloc();
-}
-
-inline void
-HttpSM::remove_ua_entry()
-{
-  vc_table.remove_entry(ua_entry);
-  ua_entry = nullptr;
-}
-
-inline void
-HttpSM::remove_server_entry()
-{
-  if (server_entry) {
-    vc_table.remove_entry(server_entry);
-    server_entry = nullptr;
-  }
-}
-
 inline int
 HttpSM::write_response_header_into_buffer(HTTPHdr *h, MIOBuffer *b)
 {
