@@ -94,34 +94,27 @@ struct ChunkedHandler {
 
   enum Action { ACTION_DOCHUNK = 0, ACTION_DECHUNK, ACTION_PASSTHRU, ACTION_UNSET };
 
-  Action action;
+  Action action = ACTION_UNSET;
 
-  IOBufferReader *chunked_reader;
-  MIOBuffer *dechunked_buffer;
-  int64_t dechunked_size;
+  IOBufferReader *chunked_reader = nullptr;
+  MIOBuffer *dechunked_buffer    = nullptr;
+  int64_t dechunked_size         = 0;
 
-  IOBufferReader *dechunked_reader;
-  MIOBuffer *chunked_buffer;
-  int64_t chunked_size;
+  IOBufferReader *dechunked_reader = nullptr;
+  MIOBuffer *chunked_buffer        = nullptr;
+  int64_t chunked_size             = 0;
 
-  /** When passing through chunked content, filter out chunked trailers.
-   *
-   * @note this is only true when: (1) we are passing through chunked content
-   * and (2) we are configured to filter out chunked trailers.
-   */
-  bool drop_chunked_trailers = false;
+  bool truncation    = false;
+  int64_t skip_bytes = 0;
 
-  bool truncation;
-  int64_t skip_bytes;
+  ChunkedState state     = CHUNK_READ_CHUNK;
+  int64_t cur_chunk_size = 0;
+  int64_t bytes_left     = 0;
+  int last_server_event  = VC_EVENT_NONE;
 
-  ChunkedState state;
-  int64_t cur_chunk_size;
-  int64_t cur_chunk_bytes_left;
-  int last_server_event;
-
-  // Chunked header size parsing info.
-  int running_sum;
-  int num_digits;
+  // Parsing Info
+  int running_sum = 0;
+  int num_digits  = 0;
 
   /// @name Output data.
   //@{
@@ -132,12 +125,12 @@ struct ChunkedHandler {
   /// It holds the header for a maximal sized chunk which will cover
   /// almost all output chunks.
   char max_chunk_header[16];
-  int max_chunk_header_len;
+  int max_chunk_header_len = 0;
   //@}
   ChunkedHandler();
 
-  void init(IOBufferReader *buffer_in, HttpTunnelProducer *p, bool drop_chunked_trailers);
-  void init_by_action(IOBufferReader *buffer_in, Action action, bool drop_chunked_trailers);
+  void init(IOBufferReader *buffer_in, HttpTunnelProducer *p);
+  void init_by_action(IOBufferReader *buffer_in, Action action);
   void clear();
 
   /// Set the max chunk @a size.
@@ -153,30 +146,28 @@ private:
   void read_chunk();
   void read_trailer();
   int64_t transfer_bytes();
-
-  constexpr static std::string_view FINAL_CRLF = "\r\n";
 };
 
 struct HttpTunnelConsumer {
   HttpTunnelConsumer();
 
   LINK(HttpTunnelConsumer, link);
-  HttpTunnelProducer *producer;
-  HttpTunnelProducer *self_producer;
+  HttpTunnelProducer *producer      = nullptr;
+  HttpTunnelProducer *self_producer = nullptr;
 
-  HttpTunnelType_t vc_type;
-  VConnection *vc;
-  IOBufferReader *buffer_reader;
-  HttpConsumerHandler vc_handler;
-  VIO *write_vio;
+  HttpTunnelType_t vc_type       = HT_HTTP_CLIENT;
+  VConnection *vc                = nullptr;
+  IOBufferReader *buffer_reader  = nullptr;
+  HttpConsumerHandler vc_handler = nullptr;
+  VIO *write_vio                 = nullptr;
 
-  int64_t skip_bytes;    // bytes to skip at beginning of stream
-  int64_t bytes_written; // total bytes written to the vc
-  int handler_state;     // state used the handlers
+  int64_t skip_bytes    = 0; // bytes to skip at beginning of stream
+  int64_t bytes_written = 0; // total bytes written to the vc
+  int handler_state     = 0; // state used the handlers
 
-  bool alive;
-  bool write_success;
-  const char *name;
+  bool alive         = false;
+  bool write_success = false;
+  const char *name   = nullptr;
 
   /** Check if this consumer is downstream from @a vc.
       @return @c true if any producer in the tunnel eventually feeds
@@ -193,37 +184,37 @@ struct HttpTunnelProducer {
   HttpTunnelProducer();
 
   DLL<HttpTunnelConsumer> consumer_list;
-  HttpTunnelConsumer *self_consumer;
-  VConnection *vc;
-  HttpProducerHandler vc_handler;
-  VIO *read_vio;
-  MIOBuffer *read_buffer;
-  IOBufferReader *buffer_start;
-  HttpTunnelType_t vc_type;
+  HttpTunnelConsumer *self_consumer = nullptr;
+  VConnection *vc                   = nullptr;
+  HttpProducerHandler vc_handler    = nullptr;
+  VIO *read_vio                     = nullptr;
+  MIOBuffer *read_buffer            = nullptr;
+  IOBufferReader *buffer_start      = nullptr;
+  HttpTunnelType_t vc_type          = HT_HTTP_SERVER;
 
   ChunkedHandler chunked_handler;
-  TunnelChunkingAction_t chunking_action;
+  TunnelChunkingAction_t chunking_action = TCA_PASSTHRU_DECHUNKED_CONTENT;
 
-  bool do_chunking;
-  bool do_dechunking;
-  bool do_chunked_passthru;
+  bool do_chunking         = false;
+  bool do_dechunking       = false;
+  bool do_chunked_passthru = false;
 
-  int64_t init_bytes_done; // bytes passed in buffer
-  int64_t nbytes;          // total bytes (client's perspective)
-  int64_t ntodo;           // what this vc needs to do
-  int64_t bytes_read;      // total bytes read from the vc
-  int handler_state;       // state used the handlers
-  int last_event;          ///< Tracking for flow control restarts.
+  int64_t init_bytes_done = 0; // bytes passed in buffer
+  int64_t nbytes          = 0; // total bytes (client's perspective)
+  int64_t ntodo           = 0; // what this vc needs to do
+  int64_t bytes_read      = 0; // total bytes read from the vc
+  int handler_state       = 0; // state used the handlers
+  int last_event          = 0; ///< Tracking for flow control restarts.
 
-  int num_consumers;
+  int num_consumers = 0;
 
-  bool alive;
-  bool read_success;
+  bool alive        = false;
+  bool read_success = false;
   /// Flag and pointer for active flow control throttling.
   /// If this is set, it points at the source producer that is under flow control.
   /// If @c NULL then data flow is not being throttled.
-  HttpTunnelProducer *flow_control_source;
-  const char *name;
+  HttpTunnelProducer *flow_control_source = nullptr;
+  const char *name                        = nullptr;
 
   /** Get the largest number of bytes any consumer has not consumed.
       Use @a limit if you only need to check if the backlog is at least @a limit.
@@ -270,9 +261,9 @@ class HttpTunnel : public Continuation
     // Default value for high and low water marks.
     static uint64_t const DEFAULT_WATER_MARK = 1 << 16;
 
-    uint64_t high_water; ///< Buffered data limit - throttle if more than this.
-    uint64_t low_water;  ///< Unthrottle if less than this buffered.
-    bool enabled_p;      ///< Flow control state (@c false means disabled).
+    uint64_t high_water;    ///< Buffered data limit - throttle if more than this.
+    uint64_t low_water;     ///< Unthrottle if less than this buffered.
+    bool enabled_p = false; ///< Flow control state (@c false means disabled).
 
     /// Default constructor.
     FlowControl();
@@ -296,19 +287,7 @@ public:
   HttpTunnelProducer *add_producer(VConnection *vc, int64_t nbytes, IOBufferReader *reader_start, HttpProducerHandler sm_handler,
                                    HttpTunnelType_t vc_type, const char *name);
 
-  /// A named variable for the @a drop_chunked_trailers parameter to @a set_producer_chunking_action.
-  static constexpr bool DROP_CHUNKED_TRAILERS = true;
-
-  /** Configure how the producer should behave with chunked content.
-   * @param[in] p Producer to configure.
-   * @param[in] skip_bytes Number of bytes to skip at the beginning of the stream (typically the headers).
-   * @param[in] action Action to take with the chunked content.
-   * @param[in] drop_chunked_trailers If @c true, chunked trailers are filtered
-   *   out. Logically speaking, this is only applicable when proxying chunked
-   *   content, thus only when @a action is @c TCA_PASSTHRU_CHUNKED_CONTENT.
-   */
-  void set_producer_chunking_action(HttpTunnelProducer *p, int64_t skip_bytes, TunnelChunkingAction_t action,
-                                    bool drop_chunked_trailers);
+  void set_producer_chunking_action(HttpTunnelProducer *p, int64_t skip_bytes, TunnelChunkingAction_t action);
   /// Set the maximum (preferred) chunk @a size of chunked output for @a producer.
   void set_producer_chunking_size(HttpTunnelProducer *producer, int64_t size);
 
@@ -376,9 +355,6 @@ private:
 private:
   int reentrancy_count = 0;
   bool call_sm         = false;
-
-  /// Corresponds to proxy.config.http.drop_chunked_trailers having a value of 1.
-  bool http_drop_chunked_trailers = false;
 };
 
 // void HttpTunnel::abort_cache_write_finish_others
@@ -508,7 +484,7 @@ HttpTunnel::get_consumer(VIO *vio)
 {
   if (vio) {
     for (int i = 0; i < MAX_CONSUMERS; i++) {
-      if (consumers[i].alive && (consumers[i].write_vio == vio || consumers[i].vc == vio->vc_server)) {
+      if (consumers[i].alive && consumers[i].write_vio == vio) {
         return consumers + i;
       }
     }
@@ -570,7 +546,7 @@ inline bool
 HttpTunnelConsumer::is_downstream_from(VConnection *vc)
 {
   HttpTunnelProducer *p = producer;
-  HttpTunnelConsumer *c;
+
   while (p) {
     if (p->vc == vc) {
       return true;
@@ -578,7 +554,8 @@ HttpTunnelConsumer::is_downstream_from(VConnection *vc)
     // The producer / consumer chain can contain a cycle in the case
     // of a blind tunnel so give up if we find ourself (the original
     // consumer).
-    c = p->self_consumer;
+    HttpTunnelConsumer *c = p->self_consumer;
+
     p = (c && c != this) ? c->producer : nullptr;
   }
   return false;
@@ -628,4 +605,4 @@ HttpTunnelProducer::unthrottle()
   }
 }
 
-inline HttpTunnel::FlowControl::FlowControl() : high_water(DEFAULT_WATER_MARK), low_water(DEFAULT_WATER_MARK), enabled_p(false) {}
+inline HttpTunnel::FlowControl::FlowControl() : high_water(DEFAULT_WATER_MARK), low_water(DEFAULT_WATER_MARK) {}
