@@ -23,7 +23,10 @@
 
 #include "HTTP2.h"
 #include "HPACK.h"
+
 #include "tscore/ink_assert.h"
+#include "tscpp/util/LocalBuffer.h"
+
 #include "records/P_RecCore.h"
 #include "records/P_RecProcess.h"
 
@@ -416,94 +419,100 @@ http2_parse_window_update(IOVec iov, uint32_t &size)
 ParseResult
 http2_convert_header_from_2_to_1_1(HTTPHdr *headers)
 {
-  MIMEField *field;
-
   ink_assert(http_hdr_type_get(headers->m_http) != HTTP_TYPE_UNKNOWN);
 
+  // HTTP Version
+  headers->version_set(HTTPVersion(1, 1));
+
   if (http_hdr_type_get(headers->m_http) == HTTP_TYPE_REQUEST) {
-    const char *scheme, *authority, *path;
-    int scheme_len, authority_len, path_len;
+    // :scheme
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME);
+        field != nullptr && field->value_is_valid(is_control_BIT | is_ws_BIT)) {
+      int scheme_len;
+      const char *scheme = field->value_get(&scheme_len);
+      const char *scheme_wks;
 
-    // Get values of :scheme, :authority and :path to assemble requested URL
-    if ((field = headers->field_find(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME)) != nullptr && field->value_is_valid()) {
-      scheme = field->value_get(&scheme_len);
+      int scheme_wks_idx = hdrtoken_tokenize(scheme, scheme_len, &scheme_wks);
+
+      if (!(scheme_wks_idx > 0 && hdrtoken_wks_to_token_type(scheme_wks) == HDRTOKEN_TYPE_SCHEME)) {
+        // unkown scheme, validate the scheme
+        if (!validate_scheme({scheme, static_cast<size_t>(scheme_len)})) {
+          return PARSE_RESULT_ERROR;
+        }
+      }
+
+      url_scheme_set(headers->m_heap, headers->m_http->u.req.m_url_impl, scheme, scheme_wks_idx, scheme_len, true);
+
+      headers->field_delete(field);
     } else {
       return PARSE_RESULT_ERROR;
     }
 
-    if ((field = headers->field_find(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY)) != nullptr && field->value_is_valid()) {
-      authority = field->value_get(&authority_len);
+    // :authority
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
+        field != nullptr && field->value_is_valid(is_control_BIT | is_ws_BIT)) {
+      int authority_len;
+      const char *authority = field->value_get(&authority_len);
+
+      url_host_set(headers->m_heap, headers->m_http->u.req.m_url_impl, authority, authority_len, true);
+
+      headers->field_delete(field);
     } else {
       return PARSE_RESULT_ERROR;
     }
 
-    if ((field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH)) != nullptr && field->value_is_valid()) {
-      path = field->value_get(&path_len);
+    // :path
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH);
+        field != nullptr && field->value_is_valid(is_control_BIT | is_ws_BIT)) {
+      int path_len;
+      const char *path = field->value_get(&path_len);
+
+      // cut first '/' if there, because `url_print()` add '/' before printing path
+      if (path_len >= 1 && path[0] == '/') {
+        ++path;
+        --path_len;
+      }
+
+      url_path_set(headers->m_heap, headers->m_http->u.req.m_url_impl, path, path_len, true);
+
+      headers->field_delete(field);
     } else {
       return PARSE_RESULT_ERROR;
     }
 
-    // Parse URL
-    Arena arena;
-    size_t url_length     = scheme_len + 3 + authority_len + path_len;
-    char *url             = arena.str_alloc(url_length);
-    const char *url_start = url;
-
-    memcpy(url, scheme, scheme_len);
-    memcpy(url + scheme_len, "://", 3);
-    memcpy(url + scheme_len + 3, authority, authority_len);
-    memcpy(url + scheme_len + 3 + authority_len, path, path_len);
-    url_parse(headers->m_heap, headers->m_http->u.req.m_url_impl, &url_start, url + url_length, true);
-    arena.str_free(url);
-
-    // Get value of :method
-    if ((field = headers->field_find(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD)) != nullptr && field->value_is_valid()) {
+    // :method
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD);
+        field != nullptr && field->value_is_valid(is_control_BIT | is_ws_BIT)) {
       int method_len;
       const char *method = field->value_get(&method_len);
 
-      int method_wks_idx = hdrtoken_tokenize(method, method_len);
-      http_hdr_method_set(headers->m_heap, headers->m_http, method, method_wks_idx, method_len, false);
+      headers->method_set(method, method_len);
+      headers->field_delete(field);
     } else {
       return PARSE_RESULT_ERROR;
     }
 
     // Combine Cookie headers ([RFC 7540] 8.1.2.5.)
-    field = headers->field_find(MIME_FIELD_COOKIE, MIME_LEN_COOKIE);
-    if (field) {
+    if (MIMEField *field = headers->field_find(MIME_FIELD_COOKIE, MIME_LEN_COOKIE); field != nullptr) {
       headers->field_combine_dups(field, true, ';');
     }
-
-    // Convert HTTP version to 1.1
-    int32_t version = HTTP_VERSION(1, 1);
-    http_hdr_version_set(headers->m_http, version);
-
-    // Remove HTTP/2 style headers
-    headers->field_delete(HTTP2_VALUE_SCHEME, HTTP2_LEN_SCHEME);
-    headers->field_delete(HTTP2_VALUE_METHOD, HTTP2_LEN_METHOD);
-    headers->field_delete(HTTP2_VALUE_AUTHORITY, HTTP2_LEN_AUTHORITY);
-    headers->field_delete(HTTP2_VALUE_PATH, HTTP2_LEN_PATH);
   } else {
-    // Set HTTP Version 1.1
-    int32_t version = HTTP_VERSION(1, 1);
-    http_hdr_version_set(headers->m_http, version);
-
     // Set status from :status
-    if ((field = headers->field_find(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS)) != nullptr) {
+    if (MIMEField *field = headers->field_find(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS); field != nullptr) {
       int status_len;
       const char *status = field->value_get(&status_len);
+
       headers->status_set(http_parse_status(status, status + status_len));
+      headers->field_delete(field);
     } else {
       return PARSE_RESULT_ERROR;
     }
-
-    // Remove HTTP/2 style headers
-    headers->field_delete(HTTP2_VALUE_STATUS, HTTP2_LEN_STATUS);
   }
 
   // Check validity of all names and values
   MIMEFieldIter iter;
   for (auto *mf = headers->iter_get_first(&iter); mf != nullptr; mf = headers->iter_get_next(&iter)) {
-    if (!mf->name_is_valid() || !mf->value_is_valid()) {
+    if (!mf->name_is_valid(is_control_BIT | is_ws_BIT) || !mf->value_is_valid()) {
       return PARSE_RESULT_ERROR;
     }
   }
@@ -590,12 +599,12 @@ http2_convert_header_from_1_1_to_2(HTTPHdr *headers)
       const char *value = headers->host_get(&value_len);
 
       if (headers->is_port_in_header()) {
-        int port            = headers->port_get();
-        char *host_and_port = static_cast<char *>(ats_malloc(value_len + 8));
+        int port = headers->port_get();
+        ts::LocalBuffer<char> buf(value_len + 8);
+        char *host_and_port = buf.data();
         value_len           = snprintf(host_and_port, value_len + 8, "%.*s:%d", value_len, value, port);
 
         field->value_set(headers->m_heap, headers->m_mime, host_and_port, value_len);
-        ats_free(host_and_port);
       } else {
         field->value_set(headers->m_heap, headers->m_mime, value, value_len);
       }
@@ -608,12 +617,13 @@ http2_convert_header_from_1_1_to_2(HTTPHdr *headers)
     if (MIMEField *field = headers->field_find(HTTP2_VALUE_PATH, HTTP2_LEN_PATH); field != nullptr) {
       int value_len;
       const char *value = headers->path_get(&value_len);
-      char *path        = static_cast<char *>(ats_malloc(value_len + 1));
-      path[0]           = '/';
+
+      ts::LocalBuffer<char> buf(value_len + 1);
+      char *path = buf.data();
+      path[0]    = '/';
       memcpy(path + 1, value, value_len);
 
       field->value_set(headers->m_heap, headers->m_mime, path, value_len + 1);
-      ats_free(path);
     } else {
       ink_abort("initialize HTTP/2 pseudo-headers");
       return PARSE_RESULT_ERROR;
@@ -924,143 +934,3 @@ http2_init()
   http2_connection_specific_headers[3] = {MIME_FIELD_TRANSFER_ENCODING, MIME_LEN_TRANSFER_ENCODING};
   http2_connection_specific_headers[4] = {MIME_FIELD_UPGRADE, MIME_LEN_UPGRADE};
 }
-
-#if TS_HAS_TESTS
-
-void forceLinkRegressionHPACK();
-void
-forceLinkRegressionHPACKCaller()
-{
-  forceLinkRegressionHPACK();
-}
-
-#include "tscore/TestBox.h"
-
-/***********************************************************************************
- *                                                                                 *
- *                       Regression test for HTTP/2                                *
- *                                                                                 *
- ***********************************************************************************/
-
-const static struct {
-  uint8_t ftype;
-  uint8_t fflags;
-  bool valid;
-} http2_frame_flags_test_case[] = {{HTTP2_FRAME_TYPE_DATA, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x08, true},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_DATA, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x04, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x08, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x20, true},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_HEADERS, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_PRIORITY, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_RST_STREAM, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_SETTINGS, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x04, true},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x08, true},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_PUSH_PROMISE, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_PING, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_PING, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_PING, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_GOAWAY, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x04, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_WINDOW_UPDATE, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x01, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x02, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x04, true},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x08, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x10, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x20, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x40, false},
-                                   {HTTP2_FRAME_TYPE_CONTINUATION, 0x80, false},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x00, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x01, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x02, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x04, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x08, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x10, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x20, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x40, true},
-                                   {HTTP2_FRAME_TYPE_MAX, 0x80, true}};
-
-static const uint8_t HTTP2_FRAME_FLAGS_MASKS[HTTP2_FRAME_TYPE_MAX] = {
-  HTTP2_FLAGS_DATA_MASK,          HTTP2_FLAGS_HEADERS_MASK,      HTTP2_FLAGS_PRIORITY_MASK, HTTP2_FLAGS_RST_STREAM_MASK,
-  HTTP2_FLAGS_SETTINGS_MASK,      HTTP2_FLAGS_PUSH_PROMISE_MASK, HTTP2_FLAGS_PING_MASK,     HTTP2_FLAGS_GOAWAY_MASK,
-  HTTP2_FLAGS_WINDOW_UPDATE_MASK, HTTP2_FLAGS_CONTINUATION_MASK,
-};
-
-REGRESSION_TEST(HTTP2_FRAME_FLAGS)(RegressionTest *t, int, int *pstatus)
-{
-  TestBox box(t, pstatus);
-  box = REGRESSION_TEST_PASSED;
-
-  for (auto i : http2_frame_flags_test_case) {
-    box.check((i.ftype >= HTTP2_FRAME_TYPE_MAX || (i.fflags & ~HTTP2_FRAME_FLAGS_MASKS[i.ftype]) == 0) == i.valid,
-              "Validation of frame flags (type: %d, flags: %d) are expected %d, but not", i.ftype, i.fflags, i.valid);
-  }
-}
-
-#endif /* TS_HAS_TESTS */
