@@ -25,7 +25,7 @@
 ** SUCH DAMAGE.
 */
 
-#include "ts/ink_config.h"
+#include "tscore/ink_config.h"
 
 #include <sys/epoll.h>
 #include <unistd.h>
@@ -438,7 +438,7 @@ main(int argc, char **argv)
       } else {
         extra_headers = malloc_check(65536);
         strncpy(extra_headers, argv[++argn], 65536 - 1);
-        extra_headers[65536] = '\0';
+        extra_headers[65535] = '\0';
       }
     } else if (strncmp(argv[argn], "-http_version", strlen(argv[argn])) == 0 && argn + 1 < argc) {
       http_version = argv[++argn];
@@ -484,7 +484,7 @@ main(int argc, char **argv)
   /* Initialize the connections table. */
   if (start == START_PARALLEL)
     max_connections = start_parallel;
-  connections       = (connection *)malloc_check(max_connections * sizeof(connection));
+  connections = (connection *)malloc_check(max_connections * sizeof(connection));
   for (cnum = 0; cnum < max_connections; ++cnum) {
     connections[cnum].conn_state        = CNST_FREE;
     connections[cnum].reusable          = 0;
@@ -495,7 +495,7 @@ main(int argc, char **argv)
   num_connections = max_parallel = num_ka_conns = 0;
 
   /* Initialize the HTTP status-code histogram. */
-  for (i                  = 0; i < 1000; ++i)
+  for (i = 0; i < 1000; ++i)
     http_status_counts[i] = 0;
 
   /* Initialize the statistics. */
@@ -606,11 +606,12 @@ main(int argc, char **argv)
 static void
 usage(void)
 {
-  (void)fprintf(stderr, "usage:	%s [-checksum] [-throttle] [-sequential] [-proxy host:port]\n"
-                        "		[-verbose] [-timeout secs] [-sip sip_file] [-agent user_agent]\n"
-                        "		[-cookie http_cookie] [-accept_gzip] [-http_version version_str]\n"
-                        "		[-keep_alive num_reqs_per_conn] [-unique_id]\n"
-                        "		[-max_connect_failures N] [-ignore_bytes] [ [-header str] ... ]\n",
+  (void)fprintf(stderr,
+                "usage:	%s [-checksum] [-throttle] [-sequential] [-proxy host:port]\n"
+                "		[-verbose] [-timeout secs] [-sip sip_file] [-agent user_agent]\n"
+                "		[-cookie http_cookie] [-accept_gzip] [-http_version version_str]\n"
+                "		[-keep_alive num_reqs_per_conn] [-unique_id]\n"
+                "		[-max_connect_failures N] [-ignore_bytes] [ [-header str] ... ]\n",
                 argv0);
   (void)fprintf(stderr, "	[-cipher str]\n");
   (void)fprintf(stderr, "	-parallel N | -rate N [-jitter]\n");
@@ -929,7 +930,7 @@ start_connection(struct timeval *nowP)
         cycle_slot = 0;
       if (prev_cycle_slot == cycle_slot) {
         return;
-#if 0        
+#if 0
         /* Unused right now, not sure why */
         printf("Warning: cycling through all socket slots\n");
         tmr_run(nowP);
@@ -1097,7 +1098,7 @@ start_socket(int url_num, int cnum, struct timeval *nowP)
       r = SSL_write(connections[cnum].ssl, urls[url_num].buf, urls[url_num].buf_bytes);
     else
       r = write(connections[cnum].conn_fd, urls[url_num].buf, urls[url_num].buf_bytes);
-    if (r < 0) {
+    if (r <= 0) {
       perror(urls[url_num].url_str);
       connections[cnum].reusable = 0;
       close_connection(cnum);
@@ -1178,8 +1179,7 @@ handle_connect(int cnum, struct timeval *nowP, int double_check)
 
     if (!RAND_status()) {
       unsigned char bytes[1024];
-      int i;
-      for (i     = 0; i < sizeof(bytes); ++i)
+      for (size_t i = 0; i < sizeof(bytes); ++i)
         bytes[i] = random() % 0xff;
       RAND_seed(bytes, sizeof(bytes));
     }
@@ -1216,7 +1216,7 @@ handle_connect(int cnum, struct timeval *nowP, int double_check)
     r = SSL_write(connections[cnum].ssl, urls[url_num].buf, urls[url_num].buf_bytes);
   else
     r = write(connections[cnum].conn_fd, urls[url_num].buf, urls[url_num].buf_bytes);
-  if (r < 0) {
+  if (r <= 0) {
     perror(urls[url_num].url_str);
     connections[cnum].reusable = 0;
     close_connection(cnum);
@@ -2760,9 +2760,9 @@ idle_connection(ClientData client_data, struct timeval *nowP __attribute__((unus
   int cnum;
   struct timeval tv;
   char strTime[32];
-
+  struct tm localtv;
   gettimeofday(&tv, NULL);
-  strftime(strTime, 32, "%T", localtime(&tv.tv_sec));
+  strftime(strTime, 32, "%T", localtime_r(&tv.tv_sec, &localtv));
 
   cnum                         = client_data.i;
   connections[cnum].idle_timer = (Timer *)0;
@@ -2846,7 +2846,7 @@ close_connection(int cnum)
   url_num = connections[cnum].url_num;
 
   /* Only check to update got_bytes, byte count errors and/or checksums
-     if the request was succesful (i.e. no HTTP error). */
+     if the request was successful (i.e. no HTTP error). */
   if (connections[cnum].http_status >= 0 && connections[cnum].http_status < 400) {
     if (do_checksum) {
       if (!urls[url_num].got_checksum) {
